;--------------------------------
;
; Jigsaw 86 include file
;
; Contains all cursor subroutines
;  used in Jigsaw 86
;
; Used for better organization
;  and easier editing of code
;
;--------------------------------
;
;CursorUp/CursorDown/CursorLeft/CursorRight:
; All affect the mouse-like cursor,
; moving it in the logical directions.
;
;All routines update the tray status before moving the cursor,
; this makes sure the tray displays correctly.
;
CursorUp:
 call UpdateTrayStatus
 call CUFunc
 ld a,(TrayVisible)
 or a
 jr nz,CUFunc
 ret
CUFunc:
 ld hl,CursorCoords+1
 ld a,(hl)
 or a
 ret z
 dec (hl)
 ret
CursorDown:
 call UpdateTrayStatus
 call CDFunc
 ld a,(TrayVisible)
 or a
 jr nz,CDFunc
 ret
CDFunc:
 ld hl,CursorCoords+1
 ld a,(hl)
 cp 63
 ret nc
 inc (hl)
 ret
CursorLeft:
 call UpdateTrayStatus
 call CLFunc
 ld a,(TrayVisible)
 or a
 jr nz,CLFunc
 ret
CLFunc:
 ld hl,CursorCoords
 ld a,(hl)
 or a
 ret z
 dec (hl)
 ret
CursorRight:
 call UpdateTrayStatus
 call CRFunc
 ld a,(TrayVisible)
 or a
 jr nz,CRFunc
 ret
CRFunc:
 ld hl,CursorCoords
 ld a,(hl)
 cp 127
 ret nc
 inc (hl)
 ret
;
;OutlineUp/OutlineDown/OutlineLeft/OutlineRight:
; All affect the outline cursor,
; moving it in the logical directions.
;
;Only OutlineLeft and OutlineRight update the tray status,
; and only do so when the tray is on the correct side to do so.
; This keeps the tray from being too sensitive and/or displaying at bad times.
;
OutlineUp:
 ld hl,CursorCoords+1
 ld a,(TrayVisible)
 or a
 jr nz,OutlineUpInTray
 ld a,(hl)
 or a
 ret z
OUMod1 = $+1
 sub $00
;Self-modified.
; 8 for 8x8 piece size
; 16 for 16x16 piece size
 ld (hl),a
 jp EndOutlineMove
OutlineUpInTray:
 ld a,(hl)
OUMod2 = $+1
 sub $00
;Self-modified.
; 10 for 8x8 piece size
; 17 for 16x16 piece size
 cp 8
 jr c,OUDecreaseTrayOffset
 cp 150
 jr nc,OUDecreaseTrayOffset
 ld (hl),a
 jp EndOutlineMove
OUDecreaseTrayOffset:
 ld hl,TrayOffset
 ld a,(hl)
 or a
 ret z
OUMod3 = $+1
 sub $00
;Self-modified.
; 10 for 8x8 piece size
; 17 for 16x16 piece size
 ld (hl),a
 jp EndOutlineMove
OutlineDown:
 ld hl,CursorCoords+1
 ld a,(TrayVisible)
 or a
 jr nz,OutlineDownInTray
 ld a,(hl)
ODMod1 = $+1
 add a,$00
;Self-modified.
; 8 for 8x8 piece size
; 16 for 16x16 piece size
 cp 64
 ret nc
 ld (hl),a
 jp EndOutlineMove
OutlineDownInTray:
 ld a,(hl)
ODMod2 = $+1
 add a,$00
;Self-modified.
; 10 for 8x8 piece size
; 17 for 16x16 piece size
 cp 55
 jr nc,ODIncreaseTrayOffset
 ld (hl),a
 jp EndOutlineMove
ODIncreaseTrayOffset:
 push bc
 call GetLastTraySpriteNum
 ld a,(TraySize)
 sub b
 pop bc
 ret z
 ld hl,TrayOffset
 ld a,(hl)
ODMod3 = $+1
 add a,$00
;Self-modified.
; 2 for 8x8 piece size
; 1 for 16x16 piece size
 ld (hl),a
 jp EndOutlineMove
OutlineLeft:
 ld a,(TrayVisible)
 or a
 jr nz,OutlineLeftInTray
 ld a,(TrayLocal)
 or a
 call z,UpdateTrayStatus
 ld hl,CursorCoords
 ld a,(TrayVisible)
 or a
 jr z,$+9
 ld (hl),3
 inc hl
OLMod1 = $+1
 ld (hl),$00
;Self-modified.
; 10 for 8x8 piece size
; 9 for 16x16 piece size
 jr EndOutlineMove
 ld a,(hl)
 or a
 ret z
OLMod2 = $+1
 sub $00
;Self-modified.
; 8 for 8x8 piece size
; 16 for 16x16 piece size
 ld (hl),a
 jr EndOutlineMove
OutlineLeftInTray:
 ld a,(TrayLocal)
 or a
 ld hl,CursorCoords
 jr nz,OutlineLeftInTrayRight
 ld (hl),3
 jr EndOutlineMove
OutlineLeftInTrayRight:
 ld a,(hl)
 sub 10
 ld (hl),a
 cp 104
 jr nc,EndOutlineMove
 ld (hl),96
 inc hl
 ld a,(hl)
OLMod3 = $+1
 and $00
;Self-modified.
; %11111000 for 8x8 piece size
; %11110000 for 16x16 piece size
 ld (hl),a
 xor a
 ld (TrayVisible),a
 jr EndOutlineMove
OutlineRight:
 ld a,(TrayVisible)
 or a
 jr nz,OutlineRightInTray
 ld a,(TrayLocal)
 or a
 call nz,UpdateTrayStatus
 ld hl,CursorCoords
 ld a,(TrayVisible)
 or a
 jr z,$+9
ORMod1 = $+1
 ld (hl),$00
;Self-modified.
; 107 for 8x8 piece size
; 108 for 16x16 piece size
 inc hl
ORMod2 = $+1
 ld (hl),$00
;Self-modified.
; 10 for 8x8 piece size
; 9 for 16x16 piece size
 jr EndOutlineMove
 ld a,(hl)
ORMod3 = $+1
 add a,$00
;Self-modified.
; 8 for 8x8 piece size
; 16 for 16x16 piece size
 cp 128
 ret nc
 ld (hl),a
 jr EndOutlineMove
OutlineRightInTray:
 ld a,(TrayLocal)
 or a
 ld hl,CursorCoords
 jr z,OutlineRightInTrayLeft
ORMod4 = $+1
 ld (hl),$00
;Self-modified.
; 117 for 8x8 piece size
; 108 for 16x16 piece size
 jr EndOutlineMove
OutlineRightInTrayLeft:
ORMod5 = $+1
 jr $+00
;Self-modified.
; 0 for 8x8 piece size
; 8 for 16x16 piece size
 ld a,(hl)
 add a,10
 ld (hl),a
 cp 15
 jr c,EndOutlineMove
ORMod6 = $+1
 ld (hl),$00
;Self-modified.
; 24 for 8x8 piece size
; 16 for 16x16 piece size
 inc hl
 ld a,(hl)
ORMod7 = $+1
 and $00
;Self-modified.
; %11111000 for 8x8 piece size
; %11110000 for 16x16 piece size
 ld (hl),a
 xor a
 ld (TrayVisible),a
EndOutlineMove
 ld c,OUTLINE_MOVE_COOLDOWN
 ld a,(TrayVisible)
 or a
 jr z,$+4
 ld c,OUTLINE_MOVE_COOLDOWN/2
 ld a,c
 ld (OutlineMoveCooldown),a
 ret
;
;Displays the cursor.
; Determines which animation table to use.
; For outline, it places the correct size cursor.
; For cursor, it displays the arrow.
; Both selection modes have a different speed for which the cursor animates.
;
PlaceCursor:
 ld hl,CursorCoords
 ld de,CursorTimer
 ld a,(SelectionMode)
 or a
 jr nz,PCOutline
 ld a,(de)
 inc a
 and %00111111
 ld (de),a
 ld b,(hl)
 inc hl
 ld c,(hl)
 ld hl,CursorSprite1
 ld a,(de)
 cp 32
 jr c,$+5
 ld hl,CursorSprite2
 jp GraySpriteClip
PCOutline:
 ld a,(de)
 inc a
 and %00011111
 ld (de),a
 ld b,(hl)
 inc hl
 ld c,(hl)
 ld hl,OutlineSpriteTable1
 ld a,(de)
 cp 16
 jr c,$+5
 ld hl,OutlineSpriteTable2
 ld a,(PieceSize)
 or a
 jr z,PCO16x16
 push bc
 ld a,(hl)
 inc hl
 push hl
 ld h,(hl)
 ld l,a
 call GraySprite
 pop hl
 pop bc
 inc hl
 inc hl
 inc hl
 ld a,(hl)
 inc hl
 ld h,(hl)
 ld l,a
 jp GraySprite
PCO16x16:
 push bc
 ld a,(hl)
 inc hl
 push hl
 ld h,(hl)
 ld l,a
 call GraySprite
 pop hl
 pop bc
 inc hl
 ld a,b
 add a,8
 ld b,a
 push bc
 ld a,(hl)
 inc hl
 push hl
 ld h,(hl)
 ld l,a
 call GraySprite
 pop hl
 pop bc
 inc hl
 ld a,c
 add a,8
 ld c,a
 push bc
 ld a,(hl)
 inc hl
 push hl
 ld h,(hl)
 ld l,a
 call GraySprite
 pop hl
 pop bc
 inc hl
 ld a,b
 sub 8
 ld b,a
 ld a,(hl)
 inc hl
 ld h,(hl)
 ld l,a
 jp GraySprite
