;=======================
;
; Jigsaw Puzzle Creator v1.1
;  by Cullen Sauls
;
; Contact:
;  webpage: http://www.macross2000.org/cullen
;  e-mail:  sauls@tamu.edu
;  ICQ:     11130342
;  AIM:     OrbitYFL
;
; Copyright  December 2001
;
; Source code looks best in Assembly Studio
;  Use Assembly Studio 8x project function for best results
;
;=======================

.plugin lite86

#include "ti86asm.inc"
#include "macros.h"

RunVal          = $91                   ;RLE Compression run value

NUM_HIGHSCORE_BYTES         = 56        ;number of bytes in highscore table

GrayCounter     = _textShadow           ;[1]
IntCount        = GrayCounter+1         ;[1]
TimerSecs       = IntCount+1            ;[1]
SPSave          = TimerSecs+1           ;[2]
AlterText       = SPSave+2              ;[4]; 0 = use original text | 1 = use new text
GrayStatus      = AlterText+4           ;[1]; 0 = monochrome | 1 = grayscale
NumChars        = GrayStatus+1          ;[2]; Number of chars in Description and Variable Name
ScreenUpdate    = NumChars+2            ;[1]; 0 = no update | 1 = redraw
TextCurPos      = ScreenUpdate+1        ;[2]; Cursor X position for typed text saving
NumPic          = TextCurPos+2          ;[1]; Number of pictures on calculator
CursorCoords    = NumPic+1              ;[2]
TextMode        = CursorCoords+2        ;[1]; 0 = uppercase | 1 = lowercase | 2 = nonletter
PicOffset       = TextMode+1            ;[1]
Layer1Addr      = PicOffset+1           ;[3]; Absolute address of picture file
Layer2Addr      = Layer1Addr+3          ;[3]; Absolute address of picture file
FileName        = Layer2Addr+3          ;[<=9]; Name of variable with length byte for saving
RandSeedSave    = FileName+9            ;[6]

TempNameStorage = $D748 + 4000          ;[900]; temporary for during VAT search
TempAbsStorage  = TempNameStorage+900   ;[300]; temporary for during VAT search

PicNameTable    = $8500+1               ;[900]; Enough room for 100 puzzles
;($8500) = 0
;Uses this format:
;pic name               ;[<=8]
;terminating byte (0)   ;[1]
AbsAddressTable = PicNameTable+900      ;[300]
VarText         = AbsAddressTable+1280  ;[<=8]
VarTextOffset   = VarText+8             ;[<=8]
DescText        = VarTextOffset+8       ;[<=256]
DescTextOffset  = DescText+256          ;[<=256]
Layer1Text      = DescTextOffset+256    ;[<=11];2 coord bytes, up to 8 chars, terminating byte
Layer2Text      = Layer1Text+11         ;[<=11];2 coord bytes, up to 8 chars, terminating byte
ScreenBuffer    = Layer2Text+11         ;[1024]
ScreenStore     = ScreenBuffer+1024     ;[1024]
TempMem         = ScreenStore+1024      ;[infinte]


VirtScr         = $FC00
PAGE1ADDR       = $FC00
PAGE2ADDR       = $CA00
PAGE1BUFFER     = ScreenBuffer
PAGE2BUFFER     = $CA00


.org _asm_exec_ram

    nop
    jp ProgStart
    .dw 0
    .dw ShellTitle

ShellTitle:
    .db "Jigsaw 86 Puzzle Creator v1.1",0

;VAT storage system, going backwards
;
;type byte
;l  \
;h  | addr
;a  /
;unused byte
;name length byte
;name [up to 8 bytes]
;
ProgStart:
 call _runindicoff
 call _flushallmenus
 res appTextSave,(iy+appflags)  ;don't save text to text shadow during execution
 ld hl,$CA00+1018
 ld de,RandSeedSave
 ld bc,6
 ldir
;Save random seed

CheckFile:
;Check for PIC files on calculator and build a table for them.
 xor a
 ld (PicNameTable-1),a
 ld (NumPic),a
 call _ram_page_7
 ld hl,$BFFF
 ld ix,TempNameStorage
CheckFileLoop:
 push hl
 ld de,(_PTEMP_END+1)
 xor a
 sbc hl,de
 pop hl
 jp c,DoneChecking
 ld a,(hl)
 dec hl
 cp $11
 jr z,AddPicFile
 dec hl
 dec hl
 dec hl
 dec hl
 call _ram_page_7
 ld b,(hl)
 inc b
PassName:
 dec hl
 djnz PassName
 jr CheckFileLoop
;
AddPicFile:
 ld e,(hl)
 dec hl
 ld d,(hl)
 dec hl
 ld c,(hl)
 dec hl
 dec hl
 ld b,(hl)
 dec hl
 ld a,(hl)
 ld (ix),a
 dec hl
 inc ix
 djnz $-7
 ld (ix),0
 inc ix
 push hl
 ld hl,NumPic
 ld a,(hl)
 inc (hl)
 ld l,a
 ld h,0
 add hl,hl
 ADD_HL_A
 push de
 ld de,TempAbsStorage
 add hl,de
 pop de
 ld (hl),c
 inc hl
 ld (hl),d
 inc hl
 ld (hl),e
 pop hl
 jr CheckFileLoop
DoneChecking:
 call _ram_page_1
 ld hl,TempNameStorage
 ld de,PicNameTable
 ld bc,1200
 ldir

 xor a
 ld hl,AlterText
 ld b,7
 ld (hl),a
 inc hl
 djnz $-2
 ld (VarText+2),a
 ld (DescText+2),a
 inc a
 ld (hl),a
 inc a
 inc hl
 ld (hl),a

 ld a,$2E
 inc hl
 ld (hl),a

 ld hl,$2E36
 ld (VarText),hl
 ld hl,$041C
 ld (Layer1Text),hl
 ld hl,$0429
 ld (Layer2Text),hl
 ld hl,$020F
 ld (DescText),hl

 ld hl,$1C3C
 ld (CursorCoords),hl

MainLoop:
 ld a,(ScreenUpdate)
 or a
 call nz,Refresh

 ld hl,ScreenStore
 ld de,ScreenBuffer
 ld bc,1024
 ldir

 ld a,%01111110
 out (1),a
 in a,(1)
 rra
 ld b,a
 call nc,MoveCursorDown
 ld a,b
 rra
 ld b,a
 call nc,MoveCursorLeft
 ld a,b
 rra
 ld b,a
 call nc,MoveCursorRight
 ld a,b
 rra
 call nc,MoveCursorUp

 ld a,%00111111
 out (1),a
 in a,(1)
 rla
 rla
 jr nc,ExitEditor
 rla
 jr nc,DoAction

 ld hl,CursorCoords
 ld b,(hl)
 inc hl
 ld c,(hl)
 ld hl,CursorSprite
 call GraySprite

 ld hl,ScreenBuffer
 ld de,$FC00
 ld bc,1024
 ldir

 halt
 halt

 jr MainLoop

ExitEditor:
 call _clrLCD
 ld hl,_textShadow
 ld de,_textShadow+1
 ld (hl),' '
 ld bc,128
 ldir
;Clear the screen so that weird characters do not show.
 res onInterrupt,(IY+onflags)
 set graphdraw,(IY+graphflags)
;Set flags to clean graph and so that the user does not get
; an ON error on exit.
 set appTextSave,(iy+appflags)
;Make text save to text shadow again.
 ld hl,RandSeedSave
 ld de,$CA00+1018
 ld bc,6
 ldir
;Restore random seed
 ret

DoAction:
 ld hl,ButtonTable
 ld de,CursorCoords
 ld b,8

DALoop:
 ld a,(de)
 cp (hl)
 jr c,OutOfBounds1
 inc hl
 cp (hl)
 jr nc,OutOfBounds2
 inc de
 inc hl
 ld a,(de)
 cp (hl)
 jr c,OutOfBounds3
 inc hl
 cp (hl)
 jr nc,OutOfBounds4
 inc hl
 ld a,(hl)
 inc hl
 ld h,(hl)
 ld l,a
 jp (hl)
OutOfBounds1:
 inc hl
OutOfBounds2:
 inc hl
 inc de
OutOfBounds3:
 inc hl
OutOfBounds4:
 inc hl
 inc hl
 inc hl
 dec de
 djnz DALoop

 jp MainLoop

ChangeDescription:
 ld a,1
 ld (AlterText),a
 call Refresh
 xor a
 call SetTextMode
 ld a,(TextCurPos)
 ld h,$0F
 ld l,a
 ld (_penCol),hl
CDLoop:
 call _getky
 cp K_ENTER
 jr z,CDEndInput
 cp K_DEL
 jr z,CDDeleteChar
 cp K_CLEAR
 jr z,CDClearAll
 cp K_ALPHA
 call z,ChangeTextMode
 ld b,a
 ld a,(_penCol)
 cp 122
 jr nc,CDLoop
 ld a,b
 call ReadChar
 cp -1
 jr z,CDLoop
 ld b,a
 ld hl,NumChars
 ld a,(hl)
 ld c,a
 inc (hl)
 ld hl,DescTextOffset
 ADD_HL_A
 ld (hl),b
 ld hl,UppercaseChars
 ld de,27
 ld a,(TextMode)
 or a
 jr z,$+4
 add hl,de
 dec a
 jr z,$+3
 add hl,de
 ld a,b
 ADD_HL_A
 ld d,(hl)            ;d = char
 ld hl,DescText+2
 ld a,c
 ADD_HL_A
 ld (hl),d
 inc hl
 ld (hl),0
 ld a,d
 call _vputmap
 jr CDLoop
CDEndInput:
 ld a,(_penCol)
 ld (TextCurPos),a
 ld a,1
 ld (ScreenUpdate),a
 ld a,(NumChars)
 ld (AlterText),a
 jp MainLoop
CDClearAll:
 call ClearDescBox
 xor a
 ld (NumChars),a
 ld (DescText+2),a
 jr $+15
CDDeleteChar:
 ld a,(NumChars)
 or a
 jp z,CDLoop
 call ClearDescBox
 ld hl,NumChars
 dec (hl)
 ld a,(hl)
 ld hl,DescText+2
 ADD_HL_A
 ld (hl),0
 ld hl,DescText
 call _VPUTS
 jp CDLoop
ClearDescBox:
 ld hl,PAGE1ADDR+(16*16)
 ld c,5
 xor a
 ld (hl),%10000000
 inc hl
 ld b,14
 ld (hl),a
 inc hl
 djnz $-2
 ld (hl),%00000001
 inc hl
 dec c
 jr nz,$-14
 ret

ChooseLayer1:
 call SelectFile
 ld (Layer1Addr),a
 ld (Layer1Addr+1),hl
 ld hl,Layer1Text+2
 ld a,(de)
 ld (hl),a
 inc hl
 inc de
 or a
 jr nz,$-5
 ld a,1
 ld (ScreenUpdate),a
 ld (AlterText+1),a
 jp MainLoop
ChooseLayer2:
 call SelectFile
 ld (Layer2Addr),a
 ld (Layer2Addr+1),hl
 ld hl,Layer2Text+2
 ld a,(de)
 ld (hl),a
 inc hl
 inc de
 or a
 jr nz,$-5
 ld a,1
 ld (ScreenUpdate),a
 ld (AlterText+2),a
 jp MainLoop 

MakePuzzleMonochrome:
 ld a,(GrayStatus)
 ld (ScreenUpdate),a
 xor a
 ld (GrayStatus),a
 jp MainLoop

MakePuzzleGrayscale:
 ld a,(GrayStatus)
 cpl
 add a,2
 ld (ScreenUpdate),a
 ld a,1
 ld (GrayStatus),a
 jp MainLoop

PreviewPuzzle:
 ld a,(AlterText+1)
 or a
 jp z,MainLoop
 ld a,(GrayStatus)
 or a
 jr z,$+9
 ld a,(AlterText+2)
 or a
 jp z,MainLoop
 call ClearMem
 ld hl,Layer2Addr
 ld a,(GrayStatus)
 or a
 jr nz,$+5
 ld hl,Layer1Addr
 ld de,PAGE2ADDR
 call CopyPicFile
 ld hl,Layer1Addr
 ld de,PAGE1ADDR
 call CopyPicFile
 call OpenGray
 ld b,30
 HALT_LOOP
LoopTilEnter:
 call _getky
 cp K_SECOND            ;$36
 jr nz,LoopTilEnter
 call CloseGray
 ld a,1
 ld (ScreenUpdate),a
 ld b,30
 HALT_LOOP
 jp MainLoop 

ChangeVariable:
 ld a,1
 ld (AlterText+3),a
 call Refresh
 xor a
 call SetTextMode
 ld a,(TextCurPos+1)
 ld h,54
 ld l,a
 ld (_penCol),hl
CVLoop:
 call _getky
 cp K_ENTER
 jp z,CVEndInput
 cp K_DEL
 jp z,CVDeleteChar
 cp K_CLEAR
 jp z,CVClearAll
 cp K_ALPHA
 call z,ChangeTextMode
 ld b,a
 ld a,(NumChars+1)
 cp 8
 jr z,CVLoop
 ld a,b
 call ReadChar
 cp -1
 jr z,CVLoop
 ld b,a
;Must check validity of char
 ld a,(NumChars+1)
 or a
 ld a,(TextMode)
 jr nz,NotFirstChar
;First char MUST be letter
 cp 2
 jr z,CVLoop
NotFirstChar:
;Make sure char is alpha/numeric
 ld hl,UppercaseChars
 ld de,27
 or a
 jr z,$+4
 add hl,de
 dec a
 jr z,$+3
 add hl,de
 ld a,b
 ADD_HL_A
 ld a,(hl)
 cp ' '             ;Spaces not allowed in variable names
 jr z,CVLoop
 sub 'A'
 cp 26              ;Between 'A' and 'Z'
 jr c,ValidChar
 ld a,(hl)
 sub 'a'
 cp 26              ;Between 'a' and 'z'
 jr c,ValidChar
 ld a,(hl)
 sub '0'
 cp 10              ;Between '0' and '9'
 jr nc,CVLoop
ValidChar:
;Char is valid for variable name
 ld hl,NumChars+1
 ld a,(hl)
 ld c,a
 inc (hl)
 ld hl,VarTextOffset
 ADD_HL_A
 ld (hl),b
 ld hl,UppercaseChars
 ld de,27
 ld a,(TextMode)
 or a
 jr z,$+4
 add hl,de
 dec a
 jr z,$+3
 add hl,de
 ld a,b
 ADD_HL_A
 ld d,(hl)            ;d = char
 ld hl,VarText+2
 ld a,c
 ADD_HL_A
 ld (hl),d
 inc hl
 ld (hl),0
 ld a,d
 call _vputmap
 jp CVLoop
CVEndInput:
 ld a,(_penCol)
 ld (TextCurPos+1),a
 ld a,1
 ld (ScreenUpdate),a
 ld a,(NumChars+1)
 ld (AlterText+3),a
 jp MainLoop
CVClearAll:
 call ClearVarBox
 xor a
 ld (NumChars+1),a
 ld (VarText+2),a
 jr $+15
CVDeleteChar:
 ld a,(NumChars+1)
 or a
 jp z,CVLoop
 call ClearVarBox
 ld hl,NumChars+1
 dec (hl)
 ld a,(hl)
 ld hl,VarText+2
 ADD_HL_A
 ld (hl),0
 ld hl,VarText
 call _VPUTS
 jp CVLoop
ClearVarBox:
 ld hl,(VarText)
 ld (TempMem),hl
 ld hl,TempMem+2
 ld a,' '
 ld b,48
 ld (hl),a
 inc hl
 djnz $-2
 ld (hl),0
 ld hl,TempMem
 jp _VPUTS

SavePuzzle:
 ld hl,AlterText
 ld a,(hl)
 or a
 jp z,MainLoop
 inc hl
 ld a,(hl)
 or a
 jp z,MainLoop
 inc hl
 ld a,(GrayStatus)
 or a
 jr z,$+7
 ld a,(hl)
 or a
 jp z,MainLoop
 inc hl
 ld a,(hl)
 or a
 jp z,MainLoop

 ld hl,VarText+2
 ld de,FileName
 ld a,(NumChars+1)
 ld (de),a
 inc de
 ld b,0
 ld c,a
 ldir

 ld hl,FileName-1

 rst 20h             ;call _Mov10toOP1
 rst 10h             ;call _FindSym to see if variable exists
 call _ram_page_1
 jp nc,MainLoop      ;don't save if variable exists

 ld hl,$C900
 ld (TempNameStorage),hl
 ld hl,$504A                ;HL = "PJ"
 ld (TempNameStorage+2),hl

 ld hl,VarText+2
 ld de,TempNameStorage+4
 ld a,(NumChars+1)
 inc a
 ld c,a
 ld b,0
 ldir
 ld hl,DescText+2
 ld a,(NumChars)
 inc a
 ld c,a
 ld b,0
 ldir
 ld a,(GrayStatus)
 ld (de),a
 inc de
;Header for string now written.
;Write high score table
 ld hl,HighScoreTable
 ld bc,NUM_HIGHSCORE_BYTES
 ldir
;High score table written
 push de
 ld hl,PAGE2ADDR
 call Clear
 ld hl,Layer1Addr
 ld de,PAGE2ADDR
 call CopyPicFile

 ld hl,PAGE2ADDR
 pop ix
 di
 call Compress
 ei

 ld a,(GrayStatus)
 or a
 jr z,NoSecondLayer

 ld hl,PAGE2ADDR
 call Clear
 ld hl,Layer2Addr
 ld de,PAGE2ADDR
 push ix
 call CopyPicFile
 pop ix

 ld hl,PAGE2ADDR
 di
 call Compress
 ei

NoSecondLayer:
 push ix
 ld hl,FileName-1        ;copy anything before string for
                         ; type byte
 rst 20h                 ;call _Mov10toOP1
 rst 10h                 ;call _FindSym to see if variable
                         ; exists
 pop hl
 ld de,TempNameStorage
 or a
 sbc hl,de               ;HL = length of string
 push hl

 call _createstrng       ;$472f  create string
 call _ex_ahl_bde        ;$45f3  ABS bde & ahl swapped
 call _ahl_plus_2_pg3    ;$4c3f  increase ABS ahl by two
 call _set_abs_dest_addr ;$5285  set that as destination for
                         ; block copy
 xor a                   ;AKA ld a,0
 ld hl,TempNameStorage   ;hl points to string_start
                         ;address of string is in relation
                         ; to 16 bit and ram page
 call _set_abs_src_addr  ;$4647  set that as source for
                         ; block copy
 xor a                   ;AKA ld a,0 -it's on already swapped
                         ; in page (0)
 pop hl                  ;length of string
 call _set_mm_num_bytes  ;$464f  set # of bytes to copy in
                         ; block copy
 call _mm_ldir           ;$52ed  ABS block copy

 call _ram_page_1

 ld hl,PAGE1ADDR+(16*34)+6
 ld b,6
 ld (hl),0
 inc hl
 djnz $-3

 ld hl,PuzzleSavedText
 call _VPUTS

 ld hl,$271B
 ld de,$5823
 call DrawRect
 ld hl,$261A
 ld de,$5924
 call DrawRect

 ld b,$FF
 HALT_LOOP

 ld a,1
 ld (ScreenUpdate),a
 jp MainLoop 
;
; Sub Routines
;
;
;Cursor Routines
;
MoveCursorDown:
 ld hl,CursorCoords+1
 ld a,(hl)
 cp 56
 ret z
 inc (hl)
 ret
MoveCursorLeft:
 ld hl,CursorCoords
 ld a,(hl)
 or a
 ret z
 dec (hl)
 ret
MoveCursorRight:
 ld hl,CursorCoords
 ld a,(hl)
 cp 120
 ret z
 inc (hl)
 ret
MoveCursorUp:
 ld hl,CursorCoords+1
 ld a,(hl)
 or a
 ret z
 dec (hl)
 ret
;
;Text Input Routines
;
;
;Interprets a keycode and returns an offset if valid key is pressed.
; If no char pressed, returns -1.
;
;Input:
; A = Pressed keycode
;
;Output:
; A = Return Value
;
ReadChar:
 ld hl,KeyChart
 ld bc,28
 cpir
 ld a,b
 or c
 jr nz,ValidKey
 ld a,-1
 ret
ValidKey:
 ld de,KeyChart
 scf
 sbc hl,de
 ld a,l
 ret
;
;Changes the text mode. Uppercase -> Lowercase -> Nonletter -> Uppercase
;
ChangeTextMode:
 push af
 ld a,(TextMode)
 inc a
 cp 3
 jr nz,$+3
 xor a
 call SetTextMode
 pop af
 ret
;
;Sets the text mode for input. 
;
;Input:
; A = mode to set
; (2 = uppercase | 1 = lowercase | 2 = nonletter)
;
SetTextMode:
 ld (TextMode),a
 ld c,a
 ld hl,PAGE1ADDR+14
 ld (hl),$FF
 inc hl
 ld (hl),$FF
 ld de,16
 ld a,%00000001
 ld b,7
 add hl,de
 ld (hl),a
 djnz $-2
 ld a,$E1
 add a,c
 ld hl,$1400
 ld (_curRow),hl
 jp _putc
;
;Copies a picture file to a memory location
;
;Input:
; HL -> Absolute address
; DE -> Memory to copy picture
;
CopyPicFile:
 push de
 ld a,(hl)
 inc hl
 ld e,(hl)
 inc hl
 ld d,(hl)
 ex de,hl
 call _inc_ptr_ahl
 call _inc_ptr_ahl
 call _set_abs_src_addr
 xor a
 pop hl
 call _set_abs_dest_addr
 ld hl,1024-16
 call _set_mm_num_bytes
 call _mm_ldir
 jp _ram_page_1
;
;Compresses a picture file
;
;Input: HL->data to compress
;       IX->compressed data storage
;
;Output: IX-> Address of first byte after compressed data
;
Compress:
 dec hl
 ld de,1024+1
;1024 bytes to compress
CLoop1:
 ld bc,0
;C = Char Counter, B = Repeat Counter
;A' is char
CLoop2:
 inc hl
 dec de
 ld a,d
 cp $FF
 ret z
;1024 bytes have been compressed
 or e
 jr z,CCompressLastData
;Make sure to compress last few bytes
 inc c
 ld a,c
 dec a
 jr nz,CNotNewChar
 inc b
 ex af,af'
 ld a,(hl)
 ex af,af'
 jr CLoop2
CNotNewChar:
 ex af,af'
 cp (hl)
 jr nz,CNotSameChar
 ex af,af'
 inc b
 ld a,b
 cp $FF
 jr z,CCompressData
 jr CLoop2
CCompressLastData:
 ld a,b
 or a
 ret z
;1024 bytes have been compressed
 ex af,af'
CNotSameChar:
 dec hl
 inc de
 cp RunVal
 jr z,CCompressData-1
 ex af,af'
 ld a,b
 cp 4
 jr nc,CCompressData
;Not enough repeating values to compress
 ex af,af'
 ld (ix),a
 inc ix
 djnz $-5
 jr CLoop1
 ex af,af'
CCompressData:
 ld (ix),RunVal
 inc ix
 ex af,af'
 ld (ix),a
 ex af,af'
 inc ix
 ld (ix),b
 inc ix
 jr CLoop1
;
;Lets the user select a picture file.
;
;Output:
; AHL -> Picture File
; DE  -> String name of picture
;
SelectFile:
 call Refresh
 ld hl,PAGE1ADDR+(10*16)+4
 ld de,9
 ld c,$FF
 ld b,45
 ld a,(hl)
 or %00000011
 ld (hl),a
 inc hl
 ld (hl),c
 inc hl
 ld (hl),c
 inc hl
 ld (hl),c
 inc hl
 ld (hl),c
 inc hl
 ld (hl),c
 inc hl
 ld (hl),c
 inc hl
 ld a,(hl)
 or %10000000
 ld (hl),a
 add hl,de
 djnz $-22

 ld hl,PAGE1ADDR+(19*16)+4
 ld de,9
 ld c,0
 ld b,35
 ld a,(hl)
 and %11111110
 or %00000010
 ld (hl),a
 inc hl
 ld (hl),c
 inc hl
 ld (hl),c
 inc hl
 ld (hl),c
 inc hl
 ld (hl),c
 inc hl
 ld (hl),c
 inc hl
 ld (hl),c
 inc hl
 ld a,(hl)
 or %10000000
 ld (hl),a
 add hl,de
 djnz $-24

 SETWONB
 ld hl,FileSelectText
 call _VPUTS
 SETBONW

 ld a,(NumPic)
 or a
 jr nz,PicOnCalc
;No picture files on the calc!
 ld hl,NoPicText
 call _VPUTS
 ld b,30
 HALT_LOOP
LoopTilEnter2:
 call _getky
 cp K_SECOND            ;$36
 jr nz,LoopTilEnter2
 pop hl
 ld a,1
 ld (ScreenUpdate),a
 jp MainLoop
PicOnCalc:
 ld b,a
 cp 6
 jr c,$+4
 ld b,5
 ld hl,PicNameTable
 ld de,$1328
 ld (_penCol),de
 call _vputs
 ld a,d
 add a,7
 ld d,a
 djnz $-11
 ld ix,PicNameTable
 xor a
 ld (PicOffset),a
 ld b,0
 call InvertSelection
SelectPicLoop:
 ld c,b
 ld b,14
 HALT_LOOP
 ld b,c
 ld a,%01111110
 out (1),a
 in a,(1)
 rra
 ld c,a
 call nc,MoveSelectCursorDown
 ld a,c
 rra
 rra
 rra
 call nc,MoveSelectCursorUp
 ld a,%00111111
 out (1),a
 in a,(1)
 rla
 rla
 rla
 jr c,SelectPicLoop
SelectPic:
 ld a,(PicOffset)
 add a,b
 ld b,a
 add a,a
 add a,b
 ld hl,AbsAddressTable
 ADD_HL_A
 ld a,(hl)
 inc hl
 ld d,(hl)
 inc hl
 ld e,(hl)
 ex de,hl
 push ix
 pop de
 ld b,30
 HALT_LOOP
 ret
;
;PuzzleSelectionCursorRoutines
;
MoveSelectCursorDown:
 ld a,(PicOffset)
 add a,b
 ld d,a
 ld a,(NumPic)
 dec a
 cp d
 ret z
 call InvertSelection
 ld a,b
 cp 4
 jr z,ScrollDown
 inc b
 call InvertSelection
 inc ix
 ld a,(ix)
 or a
 jr nz,$-6
 inc ix
 ret
ScrollDown:
 push bc
 ld hl,PicOffset
 inc (hl)
 inc ix
 ld a,(ix)
 or a
 jr nz,$-6
 inc ix
 ld de,PAGE1ADDR+(19*16)+5
 ld hl,PAGE1ADDR+(26*16)+5
 ld c,28
 ld b,5
 ld a,(hl)
 ld (de),a
 inc hl
 inc de
 djnz $-4
 push de
 ld de,11
 add hl,de
 pop de
 push hl
 ld hl,11
 add hl,de
 ex de,hl
 pop hl
 dec c
 jr nz,$-22
 ex de,hl
 ld de,12
 xor a
 ld b,7
 ld (hl),a
 inc hl
 ld (hl),a
 inc hl
 ld (hl),a
 inc hl
 ld (hl),a
 inc hl
 ld (hl),a
 add hl,de
 djnz $-10
 push ix
 push ix
 pop hl
 ld de,$2F28
 ld (_penCol),de
 call _vputs
 pop ix
 pop bc
 jr InvertSelection

MoveSelectCursorUp:
 ld a,(PicOffset)
 add a,b
 or a
 ret z
 call InvertSelection
 ld a,b
 or a
 jr z,ScrollUp
 dec b
 call InvertSelection
 dec ix
 dec ix
 ld a,(ix)
 or a
 jr nz,$-6
 inc ix
 ret
ScrollUp:
 push bc
 ld hl,PicOffset
 dec (hl)
 dec ix
 dec ix
 ld a,(ix)
 or a
 jr nz,$-6
 inc ix
 ld de,PAGE1ADDR+(53*16)+9
 ld hl,PAGE1ADDR+(46*16)+9
 ld c,28
 ld b,5
 ld a,(hl)
 ld (de),a
 dec hl
 dec de
 djnz $-4
 push de
 ld de,-11
 add hl,de
 pop de
 push hl
 ex de,hl
 ld de,-11
 add hl,de
 ex de,hl
 pop hl
 dec c
 jr nz,$-26
 ex de,hl
 ld de,-12
 xor a
 ld b,7
 ld (hl),a
 dec hl
 ld (hl),a
 dec hl
 ld (hl),a
 dec hl
 ld (hl),a
 dec hl
 ld (hl),a
 add hl,de
 djnz $-10
 push ix
 push ix
 pop hl
 ld de,$1328
 ld (_penCol),de
 call _vputs
 pop ix
 pop bc

;
;Inverts the current selected block
;
;Input:
; B = block number
;
InvertSelection:
 push bc
 ld a,b
 add a,a
 add a,a
 add a,a
 sub b          ;a <= 28
 add a,a        ;a <= 56
 add a,a        ;a <= 112
 add a,a        ;a <= 224
 ld l,a
 ld h,0
 add hl,hl
 ld de,PAGE1ADDR+(19*16)+4
 add hl,de
 ld de,10
 ld b,7
 ld a,(hl)
 xor %00000001
 ld (hl),a
 inc hl
 ld a,(hl)
 xor %11111111
 ld (hl),a
 inc hl
 ld a,(hl)
 xor %11111111
 ld (hl),a
 inc hl
 ld a,(hl)
 xor %11111111
 ld (hl),a
 inc hl
 ld a,(hl)
 xor %11111111
 ld (hl),a
 inc hl
 ld a,(hl)
 xor %11111111
 ld (hl),a
 inc hl
 ld a,(hl)
 xor %11111111
 ld (hl),a
 add hl,de
 djnz $-35
 pop bc
 ret
;
;Refreshes the main screen with new text.
;
Refresh:
 xor a
 ld (ScreenUpdate),a
 call _clrLCD

 ld b,9
 ld hl,RectCoords

 push bc
 ld e,(hl)
 inc hl
 ld d,(hl)
 inc hl
 ld a,(hl)
 inc hl
 push hl
 ld h,(hl)
 ld l,a
 call DrawRect
 pop hl
 inc hl
 pop bc
 djnz $-16

 ld b,5
 ld hl,MainScreenTxt
 call _VPUTS
 djnz $-3

 ld a,(AlterText)
 or a
 jr z,$+5
 ld hl,DescText
 call _VPUTS
 ld hl,OriginalTxt2
 ld a,(AlterText+1)
 or a
 jr z,$+5
 ld hl,Layer1Text
 call _VPUTS
 ld hl,OriginalTxt3
 ld a,(AlterText+2)
 or a
 jr z,$+5
 ld hl,Layer2Text
 call _VPUTS
 ld hl,OriginalTxt4
 ld a,(AlterText+3)
 or a
 jr z,$+5
 ld hl,VarText
 call _VPUTS

 ld hl,RadioSpriteUnselected
 ld bc,$4D1D
 ld a,(GrayStatus)
 or a
 jr nz,$+5
 ld hl,RadioSpriteSelected
 call PutSprite
 ld hl,RadioSpriteUnselected
 ld bc,$4D26
 ld a,(GrayStatus)
 or a
 jr z,$+5
 ld hl,RadioSpriteSelected
 call PutSprite

 ld hl,PuzzlePieceSprite
 ld bc,$381D
 call PutSprite
 ld hl,PuzzlePieceSprite+8
 ld bc,$401D
 call PutSprite
 ld hl,PuzzlePieceSprite+16
 ld bc,$4025
 call PutSprite
 ld hl,PuzzlePieceSprite+24
 ld bc,$3825
 call PutSprite
 ld hl,$FC00
 ld de,ScreenStore
 ld bc,1024
 ldir
 ret
;
;Draws a rectangle.
;
; (D,E) = Top Left Corner
; (H,L) = Bottom Right Corner
;
DrawRect:
 ld bc,$FC00
 push hl
 push de
 push hl
 ld l,e
 call FLine
 pop hl
 ld e,l
 call FLine
 pop de
 ld h,d
 call FLine
 pop hl
 ld d,h
 jp FLine
;
; Data
;
RectCoords:
 .dw $0000
 .dw $7F3F
 .dw $0000
 .dw $7F08
 .dw $000E
 .dw $7F16
 .dw $021B
 .dw $3423
 .dw $0228
 .dw $3430
 .dw $4B1B
 .dw $7D2C
 .dw $0235
 .dw $203D
 .dw $2C35
 .dw $5E3D
 .dw $6B35
 .dw $7D3D
ButtonTable:
 .db $00,$7F
 .db $0E,$16
 .dw ChangeDescription
 .db $02,$34
 .db $1B,$23
 .dw ChooseLayer1
 .db $02,$34
 .db $28,$30
 .dw ChooseLayer2
 .db $4D,$51
 .db $1D,$21
 .dw MakePuzzleMonochrome
 .db $4D,$51
 .db $26,$2A
 .dw MakePuzzleGrayscale
 .db $02,$20
 .db $35,$3D
 .dw PreviewPuzzle
 .db $2C,$5E
 .db $35,$3D
 .dw ChangeVariable
 .db $6B,$7D
 .db $35,$3D
 .dw SavePuzzle

KeyChart:
 .db K_LOG,     K_SIN,  K_COS,      K_TAN,      K_RAISE
 .db K_LN,      K_EE,   K_LEFTPAR,  K_RIGHTPAR, K_SLASH
 .db K_SQUARE,  K_7,    K_8,        K_9,        K_STAR
 .db K_COMMA,   K_4,    K_5,        K_6,        K_MINUS
 .db            K_1,    K_2,        K_3,        K_PLUS
 .db            K_0,    K_DOT,      K_SIGN

UppercaseChars:
 .db "ABCDEFGHIJKLMNOPQRSTUVWXYZ "
LowercaseChars:
 .db "abcdefghijklmnopqrstuvqxyz "
NonletterChars:
 .db "    ^  ()/ 789*,456-123+0. "
HighScoreTable:     ;Gets copied into every puzzle file
 .db "---"          ;initials
 .db "---"          ;initials
 .db "---"          ;initials
 .db "---"          ;initials
 .db "---"          ;initials
 .db "---"          ;initials
 .db "---"          ;initials
 .db "---"          ;initials
 .db 0,0,0          ;secs,mins,hours
 .db 0,0,0          ;secs,mins,hours
 .db 0,0,0          ;secs,mins,hours
 .db 0,0,0          ;secs,mins,hours
 .db 0,0,0          ;secs,mins,hours
 .db 0,0,0          ;secs,mins,hours
 .db 0,0,0          ;secs,mins,hours
 .db 0,0,0          ;secs,mins,hours
 .db 0              ;peaks
 .db 0              ;peaks
 .db 0              ;peaks
 .db 0              ;peaks
 .db 0              ;peaks
 .db 0              ;peaks
 .db 0              ;peaks
 .db 0              ;peaks
;
; Sprites
;
RadioSpriteSelected:
 .db %01110000
 .db %10001000
 .db %10101000
 .db %10001000
 .db %01110000
 .db %00000000
 .db %00000000
 .db %00000000
 .db %01110000
 .db %10001000
 .db %10101000
 .db %10001000
 .db %01110000
 .db %00000000
 .db %00000000
 .db %00000000
RadioSpriteUnselected:
 .db %01110000
 .db %10001000
 .db %10001000
 .db %10001000
 .db %01110000
 .db %00000000
 .db %00000000
 .db %00000000
 .db %01110000
 .db %10001000
 .db %10001000
 .db %10001000
 .db %01110000
 .db %00000000
 .db %00000000
 .db %00000000
;
CursorSprite:
 .db %11111100
 .db %10001000
 .db %10011000
 .db %10110100
 .db %11100010
 .db %10010001
 .db %00001010
 .db %00000100
 .db %11111100
 .db %10001000
 .db %10011000
 .db %10110100
 .db %11100010
 .db %10010001
 .db %00001010
 .db %00000100
 .db %11111100
 .db %11111000
 .db %11111000
 .db %11111100
 .db %11111110
 .db %10011111
 .db %00001110
 .db %00000100
;
PuzzlePieceSprite:
 .db %00000001
 .db %00000011
 .db %00000111
 .db %00000111
 .db %00000001
 .db %00110001
 .db %01110011
 .db %11111111

 .db %00011000
 .db %10111100
 .db %11111110
 .db %11111110
 .db %11111100
 .db %11111000
 .db %11111100
 .db %11111110

 .db %11111100
 .db %11111000
 .db %11111100
 .db %11111110
 .db %11111110
 .db %10111100
 .db %00011000
 .db %00000000

 .db %01111111
 .db %00111111
 .db %01111111
 .db %11111111
 .db %11111111
 .db %01111011
 .db %00110001
 .db %00000000
;
; Strings
;
MainScreenTxt:
 .db 1,21
 .db "Jigsaw 86 Puzzle Creator",0
 .db 28,85
 .db "Monochrome",0
 .db 37,85
 .db "Grayscale",0
 .db 54,4
 .db "Preview",0
 .db 54,109
 .db "Save",0
OriginalTxt1:
 .db 15,45
 .db "Description",0
OriginalTxt2:
 .db 28,15
 .db "Layer 1",0
OriginalTxt3:
 .db 41,15
 .db "Layer 2",0
OriginalTxt4:
 .db 54,55
 .db "Variable",0
FileSelectText:
 .db 11,46
 .db "Select File",0
NoPicText:
 .db 33,45
 .db "No Pictures",0
PuzzleSavedText:
 .db 28,41
 .db "Puzzle  Saved",0
;
#include "DispStr.h"
#include "PutSprite.h"
#include "fline.h"
;gray.h MUST be included last or addresses beyond it will be wrong
#include "gray.h"
