;---------------------------------------
;
; Jigsaw 86 include file
;
; Contains all miscellaneous subroutines
;  used in Jigsaw 86
;
; Used for better organization
;  and easier editing of main code
;
;---------------------------------------
;
;Get a random direction for the cursor to move during title screen animation.
; Fails after 256 tries.
;
;Direction for cursor to move
; 0 = up
; 1 = up-right
; 2 = right
; 3 = down-right
; 4 = down
; 5 = down-left
; 6 = left
; 7 = up-left
;
GetRandomDirection:
 ld c,0
GRDLoop:
 inc c
 jr z,EndGRD
 ld a,r
 rra
 and 7
 cp b
 jr z,GRDLoop
 ld b,a
 ld hl,CursorCoords
 ld a,(hl)
 or a
 jr nz,$+8
 ld a,b
 dec a
 cp 3
 jr nc,GRDLoop
 ld a,(hl)
 cp 127
 jr nz,$+9
 ld a,b
 sub 5
 cp 3
 jr nc,GRDLoop
 inc hl
 ld a,(hl)
 or a
 jr nz,$+9
 ld a,b
 sub 3
 cp 3
 jr nc,GRDLoop
 ld a,(hl)
 cp 63
 jr nz,$+9
 ld a,b
 sub 2
 cp 5
 jr c,GRDLoop
EndGRD:
 ld a,b
 ld (TempMem2+2),a
 ret
;
;Moves the selection down by one for puzzle file selection.
; Scrolls if needed.
;
MenuCursorDown:
 ld a,(MenuOffset)
 add a,b
 ld c,a
 inc c
 ld a,(NumPuz)
 cp c
 jp z,MenuLoop
 inc ix
 inc ix
 inc ix
 call DrawBlock
 inc b
 ld a,b
 cp 5
 jr c,NotAtBottom
 ld hl,MENUSCREEN+(16*17)-4
 ld de,MENUSCREEN+(16*10)-4
 ld a,26
 ld bc,8
 add hl,bc
 ex de,hl
 add hl,bc
 ex de,hl 
 ldir
 dec a
 jr nz,$-10
 ld hl,MENUSCREEN+(16*37)
 ld b,7
 call DrawBorder
 ld hl,MenuOffset
 inc (hl)
 ld de,$2528
 ld (_penCol),de
 ld a,(ix+$00)
 ld h,(ix+$01)
 ld l,(ix+$02)
 ld de,StringTempStore
 call CopyStringAHL
 ld hl,StringTempStore
 call _vputs
 ld b,4
NotAtBottom:
 call DrawBlock
 ld a,(MenuOffset)
 add a,b
 push bc
 ld hl,MENUSCREEN+(16*56)
 ld b,7
 call DrawBorder
 ld a,(ix+$00)
 ld h,(ix+$01)
 ld l,(ix+$02)
 ld de,StringTempStore
 call CopyStringAHL
 call _inc_ptr_ahl
 ld de,$3802
 ld (_penCol),de
 ld de,StringTempStore
 call CopyStringAHL
 ld hl,StringTempStore
 call _vputs
 pop bc
 jp MenuLoop
;
;Moves the selection up by one for puzzle file selection.
; Scrolls if needed.
;
MenuCursorUp:
 ld a,(MenuOffset)
 add a,b
 or a
 jp z,MenuLoop
 call DrawBlock
 dec ix
 dec ix
 dec ix
 dec b
 ld a,b
 cp $FF
 jr nz,NotAtTop
 ld hl,MENUSCREEN+(16*36)+20
 ld de,MENUSCREEN+(16*43)+20
 or a
 ld a,27
 ld bc,8
 sbc hl,bc
 ex de,hl
 sbc hl,bc
 ex de,hl
 lddr
 dec a
 jr nz,$-12
 ld hl,MENUSCREEN+(16*10)
 ld b,7
 call DrawBorder
 ld hl,MenuOffset
 dec (hl)
 ld de,$0928
 ld (_penCol),de
 ld a,(ix+$00)
 ld h,(ix+$01)
 ld l,(ix+$02)
 ld de,StringTempStore
 call CopyStringAHL
 ld hl,StringTempStore
 call _vputs
 ld b,0
NotAtTop:
 call DrawBlock
 ld a,(MenuOffset)
 add a,b
 push bc
 ld hl,MENUSCREEN+(16*56)
 ld b,7
 call DrawBorder
 ld a,(ix+$00)
 ld h,(ix+$01)
 ld l,(ix+$02)
 ld de,StringTempStore
 call CopyStringAHL
 call _inc_ptr_ahl
 ld de,$3802
 ld (_penCol),de
 ld de,StringTempStore
 call CopyStringAHL
 ld hl,StringTempStore
 call _vputs
 pop bc
 jp MenuLoop
;
;--- Begin Self Modifying setup ---
;
;This starts by saving the stack pointer, then it redirects the pointer to a data table
; so that instead of reading the addressing manualy by doing HL = (HL), I simply pop the
; address off the table for manipulation.
;
;All tables of modifying addresses are in order so I only need to set the stack point to
; the first table and simply continue to pop values through entire process.
;
;This is a rare instance where this is a good thing to do, remember that when doing something
; of this nature to NEVER push something new onto the stack or you will overwrite your data.
;
;Also be sure to turn off interrupts unless using your own that you KNOW does not push anything
; onto the stack.
;
;Always make sure to restore the SP to its original location or your program will crash the
; system when you exit.
;
;Please excuse all the commenting instructions, but Assembly Studio 8x crashes if
; #if / #else / #endif statements are used.
;
SelfModify:
 di
 ld (TempMem),sp
 ld sp,SelfModificationTables
;
 ld de,PieceSelfMod8x8Table
 ld a,(PieceSize)
 or a
 jr nz,$+5
 ld de,PieceSelfMod16x16Table
 ld b,NUM_PIECE_MOD             ;Comment out if NUM_PIECE_MOD <= 1
PieceSelfModLoop:               ;"
 pop hl                         ;"
 ld a,(de)                      ;"
 ld (hl),a                      ;"
 inc de                         ;"
 djnz PieceSelfModLoop          ;"
;
;Uncomment the following if NUM_PIECE_MOD == 1
;
; pop hl
; ld a,(de)
; ld (hl),a
;
 ld de,SelectionSelfModOutlineTable
 ld a,(SelectionMode)
 or a
 jr nz,$+5
 ld de,SelectionSelfModCursorTable
 ld b,NUM_SELECTION_MOD         ;Comment out if NUM_SELECTION_MOD <= 1
SelectionSelfModLoop:           ;"
 pop hl                         ;"
 ld a,(de)                      ;"
 ld (hl),a                      ;"
 inc hl                         ;"
 inc de                         ;"
 ld a,(de)                      ;"
 ld (hl),a                      ;"
 inc de                         ;"
 djnz SelectionSelfModLoop      ;"
;
;Uncomment the following if NUM_SELECTION_MOD == 1
;
; pop hl
; ld a,(de)
; ld (hl),a
; inc hl
; inc de
; ld a,(de)
; ld (hl),a
;
 ld de,TraySelfModOffTable
 ld a,(TrayStatus)
 or a
 jr nz,$+5
 ld de,TraySelfModOnTable
;
;
;Uncomment the following if NUM_TRAY_MOD > 1
;
; ld b,NUM_TRAY_MOD
;TraySelfModLoop:
; pop hl
; ld a,(de)
; ld (hl),a
; inc de
; djnz TraySelfModLoop
;
 pop hl                         ;Comment out if NUM_TRAY_MOD !=1
 ld a,(de)                      ;"
 ld (hl),a                      ;"
;
 ld sp,(TempMem)
 ei
 ret
;
;--- End Self-Modifying Setup ---
;
;
;Resets the variable so the program knows that [2ND] has been released.
;
SecondNotHeld:
 ld c,a
 xor a
 ld (SecondHold),a
 scf
 ld a,c
 ret
;
;Allows the user to look at the original image.
; Only lets the user do this 99 times, however.
;
SneakPeak:
 ld hl,NumPeaks
 ld a,(hl)
 inc a
 cp 100
 jp z,AssembleLoop
 inc (hl)
 ld hl,OriginalPuzzle
 ld de,PAGE1BUFFER
 ld bc,2048
 ldir
 call CopyFromBuffer
SPLoop:
 call _getky
 cp K_MORE
 jr nz,SPLoop
 jp AssembleLoop
;
;Auto-Power-Down routines. Turns off the calculator after 4 minutes.
;
APDUpdate:
 ld hl,APDCompare
 ld a,(TimerSecs)
 cp (hl)
 ret z
 ld (hl),a
 inc hl
 inc (hl)
 ld a,(hl)
 cp 240
 call z,TurnOffCalc
 xor a
 ret
APDReset:
 ld hl,APDTime
 ld (hl),0
 ret
;
;Displays an exit menu.
;
;[X] F1 - EndGame
;[X] F2 - EndProgram
;[X] F3 - Save&Quit
;[X] F4 - TurnOffCalc
;[X] EXIT - EndExitLoop
;
ExitMenu:
 call StopTimer
 call _clrLCD
 call CloseGray
 ld hl,ExitMenuText
 ld b,55
 ld c,0
 call DrawFolder
 ld hl,ExitMenuText+14
 ld b,5
 call _VPUTS
 djnz $-3
EMLoop:
 call _getky
 sub K_F4               ;$32 - [F4]
 call z,TurnOffCalc
 dec a                  ;$33 - [F3]
 jr z,SaveAndQuit
 dec a                  ;$34 - [F2]
 jp z,EndProgram
 dec a                  ;$35 - [F1]
 jp z,EndGame
 sub 2                  ;$37 - [EXIT]
 jr z,EndExitMenu
 jr EMLoop
EndExitMenu:
 call OpenGray
 call StartTimer
 jp AssembleLoop
;
;Saves the current puzzle and exits the program.
;
SaveAndQuit:
 ld hl,ResumeData
 ld (hl),1
 ld hl,PuzGray
 ld de,ResumeData+10
 call _mov4b
 ld hl,TrayVisible
 call _mov6b
 ld a,(TrayLocal)
 ld (de),a
 inc de
 ld hl,TraySpriteNums
 ld bc,128
 ldir
 ld hl,PuzzleMap
 ld bc,128
 ldir
 jp EndProgram
;
;Turns off the calculator.
;
TurnOffCalc:
 ld a,1
 out (3),a
 halt
 ld a,11
 out (3),a
LoopTilOn:
 halt
 in a,(3)
 bit 3,a                        ;check the [ON] key bit
 jr z,LoopTilOn                ;[ON] still being detected
 ret
;
;Slows down auto-scroll so a human can use it. :)
;
CheckAutoScrollDelay:
 ld hl,AutoScrollDelay
 dec (hl)
 ld a,(hl)
 or a
 ret nz
 ld (hl),AUTOSCROLL_DELAY
 ret
;
;Check if the puzzle complete.
; Compares the data on the stored screen (where nothing like cursor or tray are)
; to original puzzle (stored on ram page 1).
; Since the second layer for both stored and original puzzle are right after
; the first layer, only 1 big loop is needed.
;
CheckCompletion:
 xor a
 ld (PuzzleChange),a
 ld hl,OriginalPuzzle
 ld de,PAGE1STORE
 ld bc,2048
CCLoop1:
 ld a,(de)
 cp (hl)
 ret nz
 inc de
 inc hl
 dec bc
 ld a,b
 or c
 jr nz,CCLoop1
;The comparison found no differences between the current screen and the original
 ld a,1
 ld (Together),a
 ret
;
;Takes the picture currently on the screen and cuts out sprite data.
; Also arranges sprites in the order needed for either 8x8 or 16x16.
;
;Will also setup the tilemap in ascending order from 0 to 127.
;
;8x8 order:
; 00-01-02-03-04-05....row 0
; 15-16-17-18-19-20....row 1
;
;16x16 order:
; 01-02-05-06-09-10....row 0
; 04-03-08-07-12-11....row 1
;
BreakSprites:
 ld hl,PuzzleMap+127
 ld b,128
 dec b
 ld (hl),b
 inc b
 dec hl
 djnz $-4
;
 ld ix,PuzzleSprites
 ld hl,PAGE1BUFFER
 ld a,(PieceSize)
 or a
 jr z,BreakSprites16x16
 ld c,8
BS8x8L1:
 ld b,16
BS8x8L2:
 call CutSprite
 inc hl
 djnz BS8x8L2
 ld de,16*7
 add hl,de
 dec c
 jr nz,BS8x8L1
 ret
BreakSprites16x16:
 ld c,4
BS16x16L1:
 ld b,8
BS16x16L2:
 call CutSprite
 inc hl
 call CutSprite
 push hl
 ld de,16*8
 add hl,de
 call CutSprite
 dec hl
 call CutSprite
 pop hl
 inc hl
 djnz BS16x16L2
 ld de,16*15
 add hl,de
 dec c
 jr nz,BS16x16L1
 ret
;
;Cuts an 8x8 sprite out of the screen.
;
;Input: HL -> Top row of sprite on page 1
;       IX -> Location to put sprite
;
;Output: HL,BC  Saved
;        A,DE   Destroyed
;
CutSprite:
 push bc
 push hl
 push hl
 ld de,16
 ld b,8
 ld a,(hl)
 ld (ix),a
 add hl,de
 inc ix
 djnz $-7
 pop hl
 ld de,PAGE1BUFFER
 or a
 sbc hl,de
 ld de,PAGE2BUFFER
 add hl,de
 ld de,16
 ld b,8
 ld a,(hl)
 ld (ix),a
 add hl,de
 inc ix
 djnz $-7
 ld b,8
 ld a,$FF
 ld (ix),a
 inc ix
 djnz $-5
 pop hl
 pop bc
 ret
;
;Randomizes the tilemap so that the pieces are in random spots.
;
Randomize:
RandMod1 = $+1
 ld c,$00
;Self-modified.
; %01111111 for 8x8 piece size
; %00001111 for 16x16 piece size
RandMod2 = $+1
 ld b,$00
RLoop:
 ld a,r
 and c
 ld hl,PuzzleMap
 ADD_HL_A
 ld a,r
 and c
 ld de,PuzzleMap
 ADD_DE_A
 ld a,(hl)
 push af
 ld a,(de)
 ld (hl),a
 pop af
 ld (de),a
 djnz RLoop
 ret
;
;Options Menu
;
;Allows the user to choose piece size, selection mode, and tray status.
;
OptionsMenu:
 push bc
 push ix
 call _clrLCD
 ld hl,OptionsText
 ld b,29
 ld c,0
 call DrawFolder
 ld hl,OptionsText+11
 ld b,15
 ld c,40
 call DrawFolder
 ld hl,OptionsText+23
 ld b,11
 call _VPUTS
 djnz $-3

 ld hl,MENUSCREEN+(10*16)+9
 call OptionInvBlock
 ld hl,MENUSCREEN+(19*16)+9
 call OptionInvBlock
 ld hl,MENUSCREEN+(28*16)+9
 call OptionInvBlock

 ld b,1
 ld a,(PieceSize)
 or a
 call nz,CONoVarChange
 inc b
 ld a,(SelectionMode)
 or a
 call nz,CONoVarChange
 inc b
 ld a,(TrayStatus)
 or a
 call nz,CONoVarChange

 ld b,1
 call RefreshOptionsCursor
OMLoop:
 push bc
 call _getky
 pop bc
 cp K_MORE                  ;$38 - [MORE]
 jr z,EndOptionMenu
 cp K_SECOND                ;$36 - [2ND]
 ld c,a
 call z,ChangeOption
 ld a,c
 cp K_UP                    ;$04 - [UP]
 call z,OptionCursorUp
 ld a,c
 cp K_DOWN                  ;$01 - [DOWN]
 call z,OptionCursorDown
 jr OMLoop
EndOptionMenu:
 ld hl,PieceSize
 ld de,OptionsSave
 call _mov3b
;Copy 3 bytes from (hl) to (de)
 pop ix
 pop bc
 jp MainMenu
;
;Moves the options cursor up.
; If at the top, the cursor is moved to the bottom.
;
OptionCursorUp:
 dec b
 jr nz,RefreshOptionsCursor
 ld b,3
 jr RefreshOptionsCursor
;
;Moves the options cursor down.
; If at the bottom, the cursor is moved to the top.
;
OptionCursorDown:
 inc b
 ld a,b
 cp 4
 jr nz,RefreshOptionsCursor
 ld b,1
;
;Erases the cursor and replaces it into the right spot.
;
RefreshOptionsCursor:
 push bc
 ld hl,MENUSCREEN+(10*16)
 ld de,15
 ld b,25
 ld (hl),%10000000
 inc hl
 ld a,%00001111
 and (hl)
 ld (hl),a
 add hl,de
 djnz $-8
 pop bc 
 push bc
 ld a,1
 add a,9
 djnz $-2
 ld c,a
 ld b,4
 ld hl,OptionsCursorSprite
 call PutSprite
 pop bc
 ret
;
;Changes the current option pointed to.
; Inverts the block so that the unselected option becomes seleced and vise-versa.
;
;Input: B = Option number
;
;Ouput: BC       Saved
;       A,DE,HL  Destroyed
;
ChangeOption:
 ld a,b
 dec a
 ld hl,PieceSize
 ADD_HL_A
 ld a,(hl)
 inc a
 and 1
 ld (hl),a
CONoVarChange:
 push bc
 dec b
 ld a,b
 add a,a
 add a,a
 add a,a
 add a,b
 add a,10
 add a,a
 add a,a
 ld l,a
 ld h,0
 add hl,hl
 add hl,hl
 ld de,MENUSCREEN+9
 add hl,de
 push hl
 call OptionInvBlock
 pop hl
 inc hl
 inc hl
 inc hl
 push hl
 call OptionInvBlock
 pop hl
 inc hl
 inc hl
 inc hl
 ld de,16
 ld b,7
 ld a,(hl)
 xor %10000000
 ld (hl),a
 add hl,de
 djnz $-5
 pop bc
 ret
;
;Inverts the 3-byte-by-7-row block desired.
;
;Input: HL -> Start of block in video memory.
;
;Output: A,B,DE,HL  Destroyed.
;
OptionInvBlock:
 ld de,14
 ld b,7
 ld a,(hl)
 cpl
 ld (hl),a
 inc hl
 ld a,(hl)
 cpl
 ld (hl),a
 inc hl
 ld a,(hl)
 cpl
 ld (hl),a
 add hl,de
 djnz $-12
 ret
;
;Draws a folder outline for grouping text with a title
;
;Input: HL -> Title
;       B = Number of blank lines to leave in folder
;       C = Row number for top of folder
;
;Output: All registers destroyed.
;
;Title format:
; First byte is width of title text in bytes minus 1,
; then 2 bytes for coordinates,
; then the string.
;
DrawFolder:
 push bc
 push hl
 push bc
 ld a,(hl)
 dec a
 call GetRow
 push hl
 push hl
 inc hl
 ld e,l
 ld d,h
 inc de
 ld (hl),$FF
 or a
 jr z,$+7
 ld b,0
 ld c,a
 ldir
 pop hl
 ld de,16
 add hl,de
 pop de
 ex de,hl
 ld bc,16*7
 ldir
 pop bc
 pop hl
 ld a,(hl)
 push af
 inc hl
 SETWONB
 call _VPUTS
 SETBONW
 ld b,0
 push bc
 ld hl,CurveLeft
 call PutSprite
 pop bc
 pop af
 add a,a
 add a,a
 add a,a
 add a,8
 ld b,a
 ld hl,CurveRight
 call PutSprite
 pop bc
 ld a,c
 add a,7
 ld c,a
 push bc
 push bc
 call GetRow
 ld e,l
 ld d,h
 inc de
 ld bc,15
 ld (hl),$FF
 ldir
 pop bc
 ld a,c
 inc a
 add a,b
 ld c,a
 call GetRow
 ld e,l
 ld d,h
 inc de
 ld bc,15
 ld (hl),$FF
 ldir
 pop bc
 inc c
 call GetRow
 jp DrawBorder
;
;Gets the memory location of a row number
;
;Input: C = Row number
;
;Output: HL -> First byte of desired row
;        A,BC  Saved
;        DE    Destroyed
;
GetRow:
 ld l,c
 ld h,0
 add hl,hl
 add hl,hl
 add hl,hl
 add hl,hl
 ld de,PAGE1ADDR
 add hl,de
 ret
;
;Draws a tilemap.
; Map is self-modified to fit correct piece size.
;
;Input: HL-> Map data
;
DrawMap:
 push hl
 call ClearBuffers
 pop hl
 ld bc,0
DMLoop:
 ld a,(hl)
 cp $FF
 jr z,$+12
;$FF is a setinel value meaning no piece is there.
; If there is no piece, just skip the display.
 push hl
 call GetSpriteAddress
 push bc
 call PlaceSprite
 pop bc
 pop hl
 inc hl
 ld a,b
DMMod1 = $+1
 add a,0
;Self-Modified
; 8 for 8x8 piece size
; 16 for 16x16 piece size
 ld b,a
 cp 127
 jr c,DMLoop
 ld b,0
 ld a,c
DMMod2 = $+1
 add a,0
;Self-Modified
; 8 for 8x8 piece size
; 16 for 16x16 piece size
 ld c,a
 cp 63
 jr c,DMLoop
 ret
;
;Given a sprite number, finds the correct memory location of the sprite.
; Works for finding both 8x8 and 16x16 sprites.
;
;Input: (HL) = Sprite number
;
;Output: HL -> Sprite in memory
;        BC    Saved
;        A,DE  Destroyed
;
GetSpriteAddress:
 ld a,(hl)
 add a,a
 ld l,a
 ld h,0
 add hl,hl
 add hl,hl
 push hl
 add hl,hl
 pop de
 add hl,de
GSAMod = $+1
 jr $+00
;Self-Modified
; 2 for 8x8 piece size
; 0 for 16x16 piece size
 add hl,hl
 add hl,hl
GSA8x8:
 ld de,PuzzleSprites
 add hl,de
 ret
;
;Draws a sprite, 8x8 or 16x16.
; Automatically displays correct number sprites.
; (1 for 8x8, 4 for 16x16)
;
;Input: (B,C) = (X,Y)
;       HL -> Sprite
;
;Output: Sprite shown on screen
;        All registers destroyed
;
PlaceSprite:
 ld a,(PieceSize)
 or a
 jp nz,GraySprite
 push hl
 push bc
 call GraySprite
 pop bc
 pop hl
 ld de,24
 add hl,de
 ld a,b
 add a,8
 ld b,a
 push hl
 push bc
 call GraySprite
 pop bc
 pop hl
 ld de,24
 add hl,de
 ld a,c
 add a,8
 ld c,a
 push hl
 push bc
 call GraySprite
 pop bc
 pop hl
 ld de,24
 add hl,de
 ld a,b
 sub 8
 ld b,a
 jp GraySprite
;
;Draws a 16x16 sprite.
;
;Input: (B,C) = (X,Y)
;       HL -> Sprite
;
;Output: Sprite shown on screen
;        All registers destroyed
;
PlaceSpriteBW:
 push hl
 push bc
 call PutSprite
 pop bc
 pop hl
 ld de,8
 add hl,de
 ld a,b
 add a,8
 ld b,a
 push hl
 push bc
 call PutSprite
 pop bc
 pop hl
 ld de,8
 add hl,de
 ld a,c
 add a,8
 ld c,a
 push hl
 push bc
 call PutSprite
 pop bc
 pop hl
 ld de,8
 add hl,de
 ld a,b
 sub 8
 ld b,a
 jp PutSprite
;
;Places a 1-pixel border on both sides of the screen and clears the interior.
;
;Input: HL -> Row on screen to start border
;       B = Number of rows to place
;
;Output: HL -> First byte of row after border
;        B  = 0
;
DrawBorder:
 ld (hl),%10000000
 push bc
 ld b,14
 inc hl
 ld (hl),0
 djnz $-3
 pop bc
 inc hl
 ld (hl),%00000001
 inc hl
 djnz DrawBorder
 ret
;
;Copies a string pointed to by AHL to a location pointed to by DE
;
;Input: AHL -> absolute address of description
;       DE  -> location to put string
;        (Location _must_ be on ram page 0 to prevent writing over data when pages are swapped.)
;
;Output: AHL = AHL + number of bytes in description
;        IX  Saved
;
CopyStringAHL:
 ld c,a
CSAHLLoop:
 push hl
 call _getb_ahl
 pop hl
 ld (de),a
 or a
 ld a,c
 ret z
 call _inc_ptr_ahl
 inc de
 jr CSAHLLoop
;
;Draws an inverted 64x8 pixel block.
;
;Input: B = current selection
;
;Output: BC       Saved
;        A,DE,HL  Destroyed
;
DrawBlock:
 push bc
 ld a,b
 rlca
 rlca
 rlca
 ld l,a
 ld h,0
 add hl,hl
 ld d,h
 ld e,l
 add hl,hl
 add hl,hl
 add hl,hl
 or a
 sbc hl,de
 ld de,MENUSCREEN+(16*9)+4
 add hl,de
 ld c,7
 ld de,8
DBLoop1:
 ld b,8
DBLoop2:
 ld a,(hl)
 cpl
 ld (hl),a
 inc hl
 djnz DBLoop2
 add hl,de
 dec c
 jr nz,DBLoop1
 pop bc
 ret
;
;Takes a number and inserts it into a string in decimal format.
; Only works for 1-byte numbers, but can be easily modified for 2-byte numbers.
;
;Input: A = Number
;       B = Number of digits
;       DE -> End of number in string
;
;Output: String has number placed in
;        DE -> Start of number in string - 1
;        B = 0
;        A,HL  Destroyed
;
NumToDec:
 ld l,a
 ld h,0
NTDLoop:
 call _divHLby10
 add a,'0'
 ld (de),a
 dec de
 djnz NTDLoop
 ret
;
ContrastUp:
 ld a,($C008)
 cp 31
 ret z
 inc a
 ld ($C008),a
 out (2),a
 xor a
 ret
ContrastDown:
 ld a,($C008)
 or a
 ret z
 dec a
 ld ($C008),a
 out (2),a
 xor a
 ret
;
;Screen store, restore, and double buffering routines.
;
StoreScreen:
 ld hl,PAGE1BUFFER
 ld de,PAGE1STORE
 ld bc,2048
 ldir
 ret
RestoreScreen:
 ld hl,PAGE1STORE
 ld de,PAGE1BUFFER
 ld bc,2048
 ldir
 ret
CopyFromBuffer:
 ld hl,PAGE1BUFFER
 ld de,PAGE1ADDR
 ld bc,1024
 ldir
 ld hl,PAGE2BUFFER
 ld de,PAGE2ADDR
 ld bc,1024
 ldir
 ret
;
;Reads the keyboard until [ENTER] or [SECOND] is pressed.
;
LoopTilEnter:
 call _getky
 cp K_ENTER             ;$09
 ret z
 cp K_SECOND            ;$36
 ret z
 jr LoopTilEnter
