;---------------------------------
;
; Jigsaw 86 include file
;
; Contains all piece manipulation
;  subroutines used in Jigsaw 86
;
; Used for better organization
;  and easier editing of main code
;
;---------------------------------
;
;Deselects a piece if one is selected.
;
DeselectPiece:
 ld b,a
 ld hl,NumSelected
 ld a,(hl)
 or a
 ld a,b
 ret z
 ld (hl),0
 ld a,(SelectedPieceInTray)
 or a
 ld a,b
 ret z
 push af
 ld a,(SelectedPieceOffset)
 ld hl,TraySpriteNums
 ADD_HL_A
 call InvertSpriteInMem
 pop af
 ret
;
;Selects a piece, swaps 2 pieces, and checks for buttons in tray.
;
SelectPiece:
 ld hl,SecondHold
 ld a,(hl)
 or a
 ret nz
 inc (hl)
 ld hl,CursorCoords+1
 ld a,(TrayVisible)
 or a
 jp nz,CheckTraySelect
 ld de,SelectedPieceCoords+1
 ld a,(NumSelected)
 or a
 jr z,$+5
 ld de,TempMem+1
SPMod1 = $+1
 ld c,$00
;Self-Modified
; %11111000 for 8x8 piece size
; %11110000 for 16x16 piece size
 ld a,(hl)
 and c
 ld (de),a
 dec de
SPMod2 = $+1
 jr $+00
;Self-Modified
; 0 for 8x8 piece size
; 1 for 16x16 piece size
 add a,a
SPMod3 = $+1
 jr $+00
;Self-Modified
; 1 for 8x8 piece size
; 0 for 16x16 piece size
 rrca
 ld b,a
 dec hl
 ld a,(hl)
 and c
 ld (de),a
 rrca
 rrca
 rrca
SPMod4 = $+1
 jr $+00
;Self-Modified
; 1 for 8x8 piece size
; 0 for 16x16 piece size
 rrca
 add a,b
 ld b,a
 ld a,(NumSelected)
 or a
 jr z,NewPieceSelected
 ld (PuzzleChange),a
 xor a
 ld (NumSelected),a
 ld a,(SelectedPieceInTray)
 or a
 jr nz,SwapPieceFromTrayToBoard
;
;Swapping pieces
;
; Board -> Board
;
 ld hl,PuzzleMap
 ld a,(SelectedPieceOffset)
 ADD_HL_A
 ld c,(hl)
 push hl
 ld hl,PuzzleMap
 ld a,b
 ADD_HL_A
 ld a,(hl)
 ld (hl),c
 pop hl
 ld (hl),a
 ld hl,PuzzleMap
 call DrawMap
 jp StoreScreen
;
;Swapping Piece
;
; Tray -> Board
;
SwapPieceFromTrayToBoard:
 ld hl,TraySpriteNums
 ld a,(SelectedPieceOffset)
 ADD_HL_A
 push bc
 push hl
 call InvertSpriteInMem
 pop hl
 pop bc
 ld c,(hl)
 push hl
 ld hl,PuzzleMap
 ld a,b
 ADD_HL_A
 ld a,(hl)
 ld (hl),c
 pop hl
 ld (hl),a
 cp $FF
 call z,RemovePieceFromTray
 ld hl,PuzzleMap
 call DrawMap
 jp StoreScreen
;
NewPieceSelected:
 ld a,b
 ld (SelectedPieceOffset),a
 xor a
 ld (SelectedPieceInTray),a
 inc a
 ld (NumSelected),a
 ret
;
CheckTraySelect:
 ld a,(hl)
 dec hl
CTSMod1 = $+1
 sub $00
;Self-Modified
; 10 for 8x8 piece size
; 9 for 16x16 piece size
CTSMod2 = $+1
 cp $00
;Self-Modified
; 49 for 8x8 piece size
; 51 for 16x16 piece size
 jp nc,CheckTrayButtons
 ld hl,PieceBox8x8Left
CTSMod3 = $+1
 jr $+00
;Self-Modified
; 3 for 8x8 piece size
; 0 for 16x16 piece size
 ld hl,PieceBox16x16Left
CTSMod4 = $+1
 ld b,$00
;Self-Modified
; 22 for 8x8 piece size
; 8 for 16x16 piece size
 ld a,(TrayLocal)
 or a
 jr z,$+8
 ld a,b
 ADD_HL_A
 ld b,(hl)
 inc hl
 ld c,(hl)
 inc hl
CTSLoop:
 ld de,CursorCoords
 ld a,(de)
 cp (hl)
 jr c,CTSBadX
 sub c
 jr nc,$+3
 xor a
 cp (hl)
 jr nc,CTSBadX
 inc hl
 inc de
 ld a,(de)
 cp (hl)
 jr c,CTSBadY
 sub c
 jr nc,$+3
 xor a
 cp (hl)
 jr nc,CTSBadY
 jr CTSPieceClicked
CTSBadX:
 inc hl
CTSBadY
 inc hl
 djnz CTSLoop
 ret
CTSPieceClicked:
 ld a,(TrayOffset)
 add a,b
 dec a
 ld b,a
 ld a,(NumSelected)
 or a
 jr nz,CTSPCSwapPiece
 ld a,b
 ld (SelectedPieceOffset),a
 ld hl,TraySpriteNums
 ADD_HL_A
 ld a,(hl)
 inc a
 ret z
 ld (SelectedPieceInTray),a
 ld (NumSelected),a
 jr InvertSpriteInMem
CTSPCSwapPiece:
 ld (PuzzleChange),a
 xor a
 ld (NumSelected),a
 ld a,(SelectedPieceInTray)
 or a
 jr nz,CTSPCSwapPieceFromTrayToTray
;
;Swapping Pieces
;
; Board -> Tray
;
 ld hl,TraySpriteNums
 ld a,b
 ADD_HL_A
 ld c,(hl)
 push hl
 ld hl,PuzzleMap
 ld a,(SelectedPieceOffset)
 ADD_HL_A
 ld a,(hl)
 ld (hl),c
 pop hl
 ld (hl),a
 cp $FF
 call z,RemovePieceFromTray
 ld hl,PuzzleMap
 call DrawMap
 jp StoreScreen
;
;Swapping Pieces
;
; Tray -> Tray
;
CTSPCSwapPieceFromTrayToTray:
 ld a,(SelectedPieceOffset)
 ld hl,TraySpriteNums
 ADD_HL_A
 push hl
 push bc
 call InvertSpriteInMem
 pop bc
 pop hl
 ld c,(hl)
 push hl
 ld hl,TraySpriteNums
 ld a,b
 ADD_HL_A
 ld a,(hl)
 ld (hl),c
 pop hl
 ld (hl),a
 ret
;
;Finds a sprite in memory and inverts it.
;
InvertSpriteInMem:
 call GetSpriteAddress
CTSPCMod = $+1
 ld b,$00
 ld a,(hl)
 cpl
 ld (hl),a
 inc hl
 djnz $-4
 ret
;
;Removes a piece from the tray.
; Tray also needs to be resized.
;
;Input:  HL -> empty piece slot
;
;Output: Piece removed and tray resized
;        All registers destroyed
;
RemovePieceFromTray:
 ld e,l
 ld d,h
 inc de
 ex de,hl
 ld a,(TraySize)
 dec a
 ld (TraySize),a
 or a
 jr z,$+7
 ld c,a
 ld b,0
 ldir
 ld hl,TraySpriteNums
 ADD_HL_A
 ld (hl),$FF
 ret
;
;Checks to see if the tray buttons have been clicked.
;
CheckTrayButtons:
 ld de,TrayLocal
 ld a,(de)
 or a
 jr nz,CheckTrayButtonsRight
 ld a,(hl)
 inc hl
 sub 8
 cp 6
 jr nc,Skip2LeftButtons
 ld a,(hl)
 sub 3
 cp 5
 jr c,CTBDecreaseOffset
 sub 57
 cp 4
 jr c,CTBIncreaseOffset
Skip2LeftButtons:
 ld a,(hl)
 cp 6
 ret nc
 dec hl
 ld a,(hl)
 sub 17
 cp 4
 ret nc
 ld a,1
 ld (de),a
 ret
CheckTrayButtonsRight:
 ld a,(hl)
 inc hl
 sub 106
 cp 5
 jr c,Skip1RightButton
 sub 7
 cp 7
 ret nc
 ld a,(hl)
 sub 3
 cp 6
 jr c,CTBDecreaseOffset
 sub 57
 cp 5
 jr c,CTBIncreaseOffset
 ret
Skip1RightButton:
 ld a,(hl)
 cp 7
 ret nc
 xor a
 ld (de),a
 ret
CTBDecreaseOffset:
 ld hl,TrayOffset
 ld a,(hl)
 or a
 ret z
CTBMod1 = $+1
 sub $00
 ld (hl),a
 ret
CTBIncreaseOffset:
 call GetLastTraySpriteNum
 ld a,(TraySize)
 sub b
 ret z
 ld hl,TrayOffset
 ld a,(hl)
CTBMod2 = $+1
 add a,$00
 ld (hl),a
 ret
;
;Displays an inverted piece if it is not in the tray.
;
DisplayInvertedPiece:
 ld a,(SelectedPieceInTray)
 or a
 ret nz
 ld a,(SelectedPieceCoords+1)
 rrca
 rrca
 rrca
 ld b,a
 ld hl,0
 ld de,8*16
 or a
 jr z,$+5
 add hl,de
 djnz $-1
 ld a,(SelectedPieceCoords)
 rrca
 rrca
 rrca
 ADD_HL_A
 ld de,PAGE1BUFFER
 add hl,de
 ld de,16
DIPMod1 = $+1
 ld b,$00
;Self-Modified
; 8 for 8x8 piece size
; 16 for 16x16 piece size
DIPLoop:
 push de
 push hl
 ld a,(hl)
 cpl
 ld (hl),a
 or a
 ld de,PAGE1BUFFER
 sbc hl,de
 ld de,PAGE2BUFFER
 add hl,de
 ld a,(hl)
 cpl
 ld (hl),a
DIPMod2 = $+1
 jr $+00
;Self-Modified
; 19 for 8x8 piece size
; 0 for 16x16 piece size
 pop hl
 push hl
 inc hl
 ld a,(hl)
 cpl
 ld (hl),a
 or a
 ld de,PAGE1BUFFER
 sbc hl,de
 ld de,PAGE2BUFFER
 add hl,de
 ld a,(hl)
 cpl
 ld (hl),a
 pop hl
 pop de
 add hl,de
 djnz DIPLoop
 ret
