;---------------------------------
;
; Jigsaw 86 include file
;
; Contains all tray manipulation
;  subroutines used in Jigsaw 86
;
; Used for better organization
;  and easier editing of main code
;
;---------------------------------
;
;Determines if the tray needs to be shown or hidden.
;
UpdateTrayStatus:
 ld a,(TrayStatus)
 or a
 ret nz
 ld a,(TrayVisible)
 or a
 jr nz,CheckForTrayHide
 ld a,(TrayLocal)
 or a
 ld a,(CursorCoords)
 jr nz,CheckTrayShowRight
 or a
 ret nz
 inc a
 ld (TrayVisible),a
 ret
CheckTrayShowRight:
CTSRMod1 = $+1
 jp DummyFunction
;Self-Modified
; CTSRCursor for cursor selection mode
; CTSROutline for outline selection mode
CTSRCursor:
 cp 127
 ret nz
 ld (TrayVisible),a
 ret
CTSROutline:
CTSRMod2 = $+1
 cp $00
;Self-Modified
; 120 for 8x8 piece size
; 112 for 16x16 piece size
 ret nz
 ld (TrayVisible),a
 ret
CheckForTrayHide:
 ld a,(TrayLocal)
 or a
 ld a,(CursorCoords)
 jr nz,CheckTrayHideRight
 cp 24
 ret c
 xor a
 ld (TrayVisible),a
 ret
CheckTrayHideRight:
 cp 104
 ret nc
 xor a
 ld (TrayVisible),a
 ret
;
;Changes which side the tray is on.
;
ChangeTrayOrientation:
 ld hl,TrayLocal
 ld a,(hl)
 inc a
 and %00000001
 ld (hl),a
CTOMod1 = $
 nop
 nop
;Self-Modified
; nop \ ret for cursor selection mode
; nop \ nop for outline selection mode
 ld a,(TrayVisible)
 or a
 ret z
 ld hl,CursorCoords
 ld a,(hl)
 cp 15
 ld a,3
 jr nc,$+4
CTOMod2 = $+1
 ld a,$00
;Self-Modified
; 107 for 8x8 piece size
; 108 for 16x16 piece size
 ld (hl),a
 ret
;
;Shows the tray on the screen.
;
DisplayTray:
 ld a,(TrayLocal)
 or a
 jr nz,ShowTrayRight
 xor a
 ld c,%00000001
 ld hl,PAGE1BUFFER
 ld de,14
 ld b,64
 ld (hl),a
 inc hl
 ld (hl),a
 inc hl
 ld (hl),c
 add hl,de
 djnz $-6
 ld hl,PAGE2BUFFER
 ld b,64
 ld (hl),a
 inc hl
 ld (hl),a
 inc hl
 ld (hl),c
 add hl,de
 djnz $-6
 ld bc,$0201
 ld hl,TrayArrowLeftSprite
 xor a
 call ShowTrayArrow
 ld bc,$1201
 ld hl,TrayArrowRightSprite
 ld a,1
 call ShowTrayArrow
 ld bc,$0904
 ld hl,TrayArrowUpSprite
 ld a,(TrayOffset)
 call ShowTrayArrow
 ld hl,TrayArrowDownSprite
 call GetLastTraySpriteNum
 ld a,(TraySize)
 sub b
 ld bc,$093D
 call ShowTrayArrow
 ld bc,$0309
 ld a,(PieceSize)
 or a
 jp nz,DisplayTraySprites8x8
 jr DisplayTraySprites16x16
ShowTrayRight:
 xor a
 ld c,%10000000
 ld hl,PAGE1BUFFER+13
 ld de,14
 ld b,64
 ld (hl),c
 inc hl
 ld (hl),a
 inc hl
 ld (hl),a
 add hl,de
 djnz $-6
 ld hl,PAGE2BUFFER+13
 ld b,64
 ld (hl),c
 inc hl
 ld (hl),a
 inc hl
 ld (hl),a
 add hl,de
 djnz $-6
 ld bc,$6B01
 ld hl,TrayArrowLeftSprite
 ld a,1
 call ShowTrayArrow
 ld bc,$7B01
 ld hl,TrayArrowRightSprite
 xor a
 call ShowTrayArrow
 ld bc,$7204
 ld hl,TrayArrowUpSprite
 ld a,(TrayOffset)
 call ShowTrayArrow
 ld hl,TrayArrowDownSprite
 call GetLastTraySpriteNum
 ld a,(TraySize)
 sub b
 ld bc,$723D
 call ShowTrayArrow
 ld bc,$6B09
 ld a,(PieceSize)
 or a
 jr nz,DisplayTraySprites8x8
 inc b
DisplayTraySprites16x16:
 ld a,(TrayOffset)
 ld hl,TraySpriteNums
 ADD_HL_A
 ld d,3
DTS16x16Loop:
 ld a,(hl)
 cp $FF
 ret z
 push hl
 push de
 push bc
 call GetSpriteAddress
 call PlaceSprite
 pop bc
 pop de
 pop hl
 inc hl
 ld a,c
 add a,17
 ld c,a
 dec d
 jr nz,DTS16x16Loop
 ret
DisplayTraySprites8x8:
 inc c
 ld a,(TrayOffset)
 ld hl,TraySpriteNums
 ADD_HL_A
 ld d,5
DTS8x8Loop:
 ld a,(hl)
 cp $FF
 ret z
 push bc
 push hl
 push de
 push bc
 call GetSpriteAddress
 call PlaceSprite
 pop bc
 pop de
 pop hl
 inc hl
 ld a,b
 add a,10
 ld b,a
 ld a,(hl)
 cp $FF
 jr nz,$+4
 pop bc
 ret
 push hl
 push de
 push bc
 call GetSpriteAddress
 call PlaceSprite
 pop bc
 pop de
 pop hl
 inc hl
 pop bc
 ld a,c
 add a,10
 ld c,a
 dec d
 jr nz,DTS8x8Loop
 ret
;
;Gets the offset of the very last sprite showing in the tray.
;
GetLastTraySpriteNum:
 ld a,(TraySize)
 ld c,a
 ld a,(TrayOffset)
GLTSNMod = $+1
 add a,$00
;Self-Modified
; 3 for 8x8 piece size
; 10 for 16x16 piece size
 cp c
 jr c,$+3
 ld a,c
 ld b,a
 ret
;
;Displays a tray arrow.
; Grays the arrow out if the option is not available.
;
ShowTrayArrow:
 ld de,TempMem
 push bc
 ld bc,8
 ldir
 ld hl,TempMem
 ld bc,16
 ldir
 or a
 jr nz,$+15
 ld hl,TempMem
 ld de,TempMem+1
 ld (hl),0
 ld bc,7
 ldir
 ld hl,TempMem
 pop bc
 jp GraySprite
