;=======================
;
; Jigsaw 86 v1.2
;  by Cullen Sauls
;
; Title picture: Ki Jeon Nam
;
; Contact:
;  webpage: http://www.macross2000.org/cullen
;  e-mail:  sauls@tamu.edu
;  ICQ:     11130342
;  AIM:     OrbitYFL
;
; Copyright  August 2002
;
; Source code looks best in Assembly Studio
;  Use Assembly Studio 8x project function for best results
;
;=======================

.plugin lite86

#include "ti86asm.inc"
#include "macros.h"

OUTLINE_MOVE_COOLDOWN       = 8             ;frames
AUTOSCROLL_DELAY            = 8             ;frames
;Timers for various things - designates delay times
; Put here for easy change

NUM_RESET_VARS              = 13
NUM_SET_VARS                = 2
NUM_HIGHSCORE_BYTES         = 56

VirtScr                     = $FC00
MENUSCREEN                  = $FC00
PAGE1ADDR                   = $FC00
PAGE2ADDR                   = $CA00

;Table has enough room for 256 entries, only 255 entries usable due to byte size
AbsAddrTable                = $CA00                     ;[<=$300]
;Storage system for absolute addresses
;AHL stored in 3 bytes as A,H,L
StringTempStore             = $CD00                     ;[<100]
;Strings stored here temporarily

CompressedData              = $8400                     ;[<=4096]; Compressed picture data
TempMem                     = $9600                     ;[<=128]; Mostly used for string storage
TempMem2                    = $9680                     ;[<=128]; Rarely used
TraySpriteNums              = $9700                     ;[<=128]; Sprites in tray
PuzzleMap                   = $9800                     ;[<=128]; Tilemap for puzzle
PuzzleSprites               = $9900                     ;[128 * 24]; Sprite data
PAGE1BUFFER                 = $A500                     ;[1024]
PAGE2BUFFER                 = $A900                     ;[1024]
PAGE1STORE                  = $AD00                     ;[1024]
PAGE2STORE                  = $B100                     ;[1024]
OriginalPuzzle              = $B500                     ;[2 * 1024]; Both layers

;The following are set sometime during program execution
NumPuz                      = _textShadow               ;[1]
MenuOffset                  = NumPuz+1                  ;[1]
GrayCounter                 = MenuOffset+1              ;[1]
SelectedPieceCoords         = GrayCounter+1             ;[2]
SelectedPieceInTray         = SelectedPieceCoords+2     ;[1]
SelectedPieceOffset         = SelectedPieceInTray+1     ;[1]
IntCount                    = SelectedPieceOffset+1     ;[1]
SecondHold                  = IntCount+1                ;[1]
;Stores whether or not [2ND] is beind held
HighScoreAddr               = SecondHold+1              ;[3]
RandSeedSave                = HighScoreAddr+3           ;[6]

PieceSize                   = RandSeedSave+6            ;[1]
;0 = 16x16 | 1 = 8x8
SelectionMode               = PieceSize+1               ;[1]
;0 = Cursor | 1 = Outline
TrayStatus                  = SelectionMode+1           ;[1]
;0 = On | 1 = Off

;Should be saved by save & quit
PuzGray                     = TrayStatus+1              ;[1]
CursorCoords                = PuzGray+1                 ;[2]
TraySize                    = CursorCoords+2            ;[1]
;# of pieces in tray

;The following need to be reset when a new puzzle is loaded
Together                    = TraySize+1                ;[1]
PuzzleChange                = Together+1                ;[1]
NumSelected                 = PuzzleChange+1            ;[1]
CursorTimer                 = NumSelected+1             ;[1]
APDCompare                  = CursorTimer+1             ;[1]
APDTime                     = APDCompare+1              ;[1]
OutlineMoveCooldown         = APDTime+1                 ;[1]

;Should be saved by save & quit
TrayVisible                 = OutlineMoveCooldown+1     ;[1]
;0 = False | 1 = True
TrayOffset                  = TrayVisible+1             ;[1]
TimerSecs                   = TrayOffset+1              ;[1]
TimerMins                   = TimerSecs+1               ;[1]
TimerHrs                    = TimerMins+1               ;[1]
NumPeaks                    = TimerHrs+1                ;[1]

;The following need to be set when a new puzzle is loaded
AutoScrollDelay             = NumPeaks+1                ;[1]

;Should be saved by save & quit
TrayLocal                   = AutoScrollDelay+1         ;[1]
;0 = Left | 1 = Right


.org _asm_exec_ram

    nop
    jp ProgStart
    .dw 1
    .dw ShellTitle
    .dw Icon
ShellTitle:
    .db "Jigsaw 86 v1.2 by Cullen Sauls",0
Icon:
.db 15, 2
 .db %00000001, %00011000
 .db %00000011, %10111100
 .db %00000111, %11111110
 .db %00000111, %11111110
 .db %00000001, %11111100
 .db %00110001, %11111000
 .db %01110011, %11111100
 .db %11111111, %11111110
 .db %01111111, %11111100
 .db %00111111, %11111000
 .db %01111111, %11111100
 .db %11111111, %11111110
 .db %11111111, %11111110
 .db %01111011, %10111100
 .db %00110001, %00011000
;
;Data to be written back, not compressed by Lite86 II
OptionsSave:
 .db 0,0,0
ResumeData:
 .db 0                                  ;0 = no resume | 1 = resume
 .db 0,"________"                       ;puzzle name length and name
 .db 0,0,0,0,0,0,0,0,0,0,0              ;Variables
Label:
 .dw 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0    ;Data
 .dw 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0    ;Data
 .dw 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0    ;Data
 .dw 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0    ;Data
 .dw 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0    ;Data
 .dw 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0    ;Data
 .dw 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0    ;Data
 .dw 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0    ;Data
ResumeDataEnd:
;end of noncompressed writeback data
;
;VAT storage system, going backwards
;
;type byte
;l  \
;h  | addr
;a  /
;unused byte
;name length byte
;name [up to 8 bytes]
;
ProgStart:
 call _runindicoff
 call _flushallmenus
 res appTextSave,(iy+appflags)  ;don't save text to text shadow during execution
 ld hl,$CA00+1018
 ld de,RandSeedSave
 ld bc,6
 ldir
;Save random seed

 ld a,(ResumeData)
 or a
 jp nz,ResumeGame
PuzzleDeleted:
 ld hl,ResumeData
 ld (hl),0

 ld hl,TitleString-1
 rst 20h
 rst 10h
 jp c,EndTitleScreenLoop     ;don't display the title screen if the string isn't there
 ld a,b
 ex de,hl
 call _get_word_ahl
 push de
 call _inc_ptr_ahl
 ld de,StringTempStore
 call CopyStringAHL
 call _inc_ptr_ahl
 call CopyStringAHL
 ld b,NUM_HIGHSCORE_BYTES+2
 call _inc_ptr_ahl
 djnz $-3
 call _set_abs_src_addr
 ld a,1
 ld hl,CompressedData-$8000
 call _set_abs_dest_addr
 pop de
 xor a
 ex de,hl
 call _set_mm_num_bytes
 call _mm_ldir
 call _ram_page_1

 xor a
 ld hl,NumSelected
 ld b,8
 ld (hl),a
 inc hl
 djnz $-2
 ld (TempMem2),a
;Temporary data value to store whether or not the title screen should be
; messed with.
 ld (TempMem2+1),a
;Temporary variable to store timer data during title screen manipulation
 ld hl,PieceSize
 ld (hl),a
 inc hl
 ld (hl),a
 inc hl
 inc a
 ld (hl),a
;
 ld hl,CompressedData
 ld de,PAGE1BUFFER
 call DispRLE
 call DispRLE

 ld hl,PAGE1BUFFER
 ld de,PAGE1STORE
 ld bc,2048
 ldir
 call CopyFromBuffer
 call OpenGray
 call SelfModify
 call BreakSprites
 call StartTimer
;
TitleScreenLoop:
 xor a
 ld (SecondHold),a
 ld a,(TempMem2)
 or a
 jr nz,NoSetTitleManip
 ld a,(TimerSecs)
 cp 3
 jr nz,NoSetTitleManip
 ld (TempMem2),a
 ld hl,$1C3C
 ld (CursorCoords),hl
 call GetRandomDirection
NoSetTitleManip:
 ld a,(TempMem2)
 or a
 jr z,EndManipTitle

 call RestoreScreen
 call PlaceCursor
 ld a,(NumSelected)
 or a
 call nz,DisplayInvertedPiece
 call CopyFromBuffer

 ld hl,TempMem2+1
 inc (hl)
 ld a,(hl)
 cp 30
 jr z,ChangeTitlePieces
 ld a,(TempMem2+2)
 ld b,a
 or a
 call z,CursorUp
 dec b
 jr nz,$+8
 call CursorUp
 call CursorRight
 dec b
 call z,CursorRight
 dec b
 jr nz,$+8
 call CursorDown
 call CursorRight
 dec b
 call z,CursorDown
 dec b
 jr nz,$+8
 call CursorDown
 call CursorLeft
 dec b
 call z,CursorLeft
 dec b
 jr nz,$+8
 call CursorUp
 call CursorLeft
 jr EndManipTitle
ChangeTitlePieces:
 xor a
 ld (hl),a
 call GetRandomDirection
 call SelectPiece
EndManipTitle:
 call _getky
 cp K_EXIT
 jp z,EndProgram
 or a
 jr nz,EndTitleScreenLoop
 jp TitleScreenLoop
EndTitleScreenLoop:
 call StopTimer
 call CloseGray
;
 ld hl,OptionsSave
 ld de,PieceSize
 call _mov3b
;Move 3 bytes from (hl) to (de)
;
CheckFile:
 xor a
 ld (NumPuz),a
 call _ram_page_7
 ld hl,$BFFF
 ld ix,AbsAddrTable
CheckFileLoop:
 push hl
 ld de,(_PTEMP_END+1)
 xor a
 sbc hl,de
 pop hl
 jp c,DoneChecking
 ld a,(hl)
 dec hl
 cp $0C
 jr z,CheckString
 dec hl
 dec hl
 dec hl
 dec hl
 call _ram_page_7
 ld b,(hl)
 inc b
PassName:
 dec hl
 djnz PassName
 jr CheckFileLoop
;
CheckString:
 ld e,(hl)
 dec hl
 ld d,(hl)
 dec hl
 ld a,(hl)
 dec hl
 dec hl
 ex de,hl
 call _inc_ptr_ahl
 call _inc_ptr_ahl
 call _inc_ptr_ahl
 call _inc_ptr_ahl
 ld c,a
 push de
 call _get_word_ahl
 push hl
 ld hl,$504A            ;HL = "PJ"
 or a
 sbc hl,de
 pop hl
 pop de
 jr nz,BadString
 push hl
 ld hl,NumPuz
 inc (hl)
 pop hl
 ld (ix),c
 ld (ix+$01),h
 ld (ix+$02),l
 inc ix
 inc ix
 inc ix
BadString:
 ex de,hl
 jr PassName-5

DoneChecking:
 ld ix,AbsAddrTable
 xor a
 ld (MenuOffset),a
 ld b,a
MainMenu:
;
;Setup the look for the menu screen
;
 push bc
 push ix
 call _clrLCD

 ld hl,MenuText
 ld b,37
 ld c,0
 call DrawFolder
 ld hl,MenuText+18
 ld b,7
 ld c,48
 call DrawFolder
 ld hl,MenuText+33
 call _VPUTS

 ld hl,PuzzlePieceSprite1
 ld bc,$0813
 call PlaceSpriteBW
 ld hl,PuzzlePieceSprite2
 ld bc,$6813
 call PlaceSpriteBW

 pop ix
;
;Display known puzzles and description of first one
; (5 puzzles at a time will display with scrolling)
;
 ld a,(NumPuz)
 or a
 jr nz,PuzzlesOnCalc
;
;No puzzles were detected. Inform the user and quit.
;
 pop bc
 ld hl,NoPuzzleText
 call _VPUTS
 call LoopTilEnter
 jp EndProgram
;
;Puzzles detected, allow user to select one.
;
PuzzlesOnCalc:
 pop bc
 push bc
 push ix
 push ix
 push af

 ld a,b
 or a
 jr z,$+10
 dec ix
 dec ix
 dec ix
 djnz $-6

 pop af
 ld de,$0228
 ld b,a
 cp 6
 jr c,$+4
 ld b,5
ShowPuzLoop:
 ld a,d
 add a,7
 ld d,a
 ld (_penCol),de
 ld a,(ix+$00)
 ld h,(ix+$01)
 ld l,(ix+$02)
 inc ix
 inc ix
 inc ix
 push de
 ld de,StringTempStore
 call CopyStringAHL
 ld hl,StringTempStore
 call _vputs
 pop de
 djnz ShowPuzLoop
 pop hl

 ld a,(hl)
 inc hl
 ld d,(hl)
 inc hl
 ld e,(hl)
 ex de,hl
 ld de,StringTempStore
 call CopyStringAHL
 call _inc_ptr_ahl
 ld de,StringTempStore
 call CopyStringAHL
 ld de,$3802
 ld (_penCol),de
 ld hl,StringTempStore
 call _vputs
 pop ix
 pop bc
 call DrawBlock

MenuLoop:
 push bc
 call _getky
 pop bc
 dec a                  ;$01 - [DOWN]
 jp z,MenuCursorDown
 sub $03                ;$04 - [UP]
 jp z,MenuCursorUp
 sub $32                ;$36 - [2ND]
 jr z,SelectPuz
 dec a                  ;$37 - [EXIT]
 jp z,EndProgram
 dec a                  ;$38 - [MORE]
 jp z,OptionsMenu
 jr MenuLoop
;
ResumeGame:
 ld hl,ResumeData
 rst 20h
 rst 10h
 jp c,PuzzleDeleted         ;They deleted the puzzle file, weird people...

 call _inc_ptr_bde
 call _inc_ptr_bde
 call _inc_ptr_bde
 call _inc_ptr_bde
 call _inc_ptr_bde
 call _inc_ptr_bde

 ld hl,$CA00
 ld (hl),b
 inc hl
 ld (hl),d
 inc hl
 ld (hl),e

 ld hl,OptionsSave
 ld de,PieceSize
 call _mov3b

 ld ix,$CA00
SelectPuz:
 call _clrLCD
 ld a,(ix+$00)
 ld h,(ix+$01)
 ld l,(ix+$02)
;
;Setup the resume data in case save & quit is executed
;
 ld de,ResumeData+2
 call CopyStringAHL
 ld de,ResumeData+2
 ld b,a
 ld c,0
 ld a,(de)
 or a
 jr z,$+6
 inc c
 inc de
 jr $-6
 ld a,c
 ld (ResumeData+1),a
 ld a,b
;
 call _inc_ptr_ahl
 ld de,StringTempStore
 call CopyStringAHL
 call _inc_ptr_ahl
 ld d,a
 push hl
 call _getb_ahl
 ld (PuzGray),a
 pop hl
 ld a,d
 call _inc_ptr_ahl

 ld (HighScoreAddr),a
 ld (HighScoreAddr+1),hl
 ld b,NUM_HIGHSCORE_BYTES
 call _inc_ptr_ahl
 djnz $-3

 call _set_abs_src_addr
 ld a,1
 ld hl,CompressedData-$8000
 call _set_abs_dest_addr
 xor a
 ld hl,4096
 call _set_mm_num_bytes
 call _mm_ldir
 call _ram_page_1
;
 ld hl,CompressedData
 ld de,OriginalPuzzle
 call DispRLE
 ld a,(PuzGray)
 or a
 call nz,DispRLE

 ld a,(PuzGray)
 or a
 jr nz,$+13
 ld hl,OriginalPuzzle
 ld de,OriginalPuzzle+1024
 ld bc,1024
 ldir

 ld hl,OriginalPuzzle
 ld de,PAGE1BUFFER
 ld bc,1024
 ldir
 ld hl,OriginalPuzzle+1024
 ld de,PAGE2BUFFER
 ld bc,1024
 ldir

 call CopyFromBuffer
;
 xor a
 ld b,NUM_RESET_VARS
 ld hl,Together
 ld (hl),a
 inc hl
 djnz $-2
 inc a
 ld b,NUM_SET_VARS
 ld (hl),a
 inc hl
 djnz $-2

 call SelfModify
;Self-Modify all necessary code
 call BreakSprites
 call Randomize
 ld hl,PuzzleMap
 ld de,TraySpriteNums
 ld bc,128
 ldir
StartMod1 = $+1
 jr $+00
;Self-Modified
; 0 for tray on
; 13 for tray off
 ld hl,PuzzleMap
 ld de,PuzzleMap+1
 ld (hl),$FF
 ld bc,128
 ldir
 ld hl,TraySpriteNums
StartMod2 = $+1
 ld a,$00
;Self-Modified
; 128 for 8x8 piece size
; 32 for 16x16 piece size
 ld (TraySize),a
 ADD_HL_A
 ld (hl),$FF

StartMod3 = $+1
 ld hl,$00
;Self-Modified
; $1C3C for cursor selection mode
; $0000 for outline selection mode
 ld (CursorCoords),hl
;
 ld hl,ResumeData
 ld a,(hl)
 or a
 jr z,NoResumeData
 dec a
 ld (hl),a
 ld hl,ResumeData+10
 ld de,PuzGray
 call _mov4b
 ld de,TrayVisible
 call _mov6b
 ld a,(hl)
 ld (TrayLocal),a
 inc hl
 ld de,TraySpriteNums
 ld bc,128
 ldir
 ld de,PuzzleMap
 ld bc,128
 ldir
NoResumeData:
 call OpenGray
 call LoopTilEnter
;Let user see pic before starting
 call ClearMem

 ld hl,PuzzleMap
 call DrawMap
 call StoreScreen

 call PlaceCursor
 call CopyFromBuffer
 ld a,1
 ld (SecondHold),a
;Stop multiple readings of [2ND] key
 res onInterrupt,(IY+onflags)
;Make sure the [ON] flag is not set.
 call StartTimer
AssembleLoop:
 call RestoreScreen
 call _getky
 or a
 push af
 call z,APDUpdate
 call nz,APDReset
 pop af

 sub K_PLUS                     ;$0A - [+]
 call z,ContrastUp
 dec a                          ;$0B - [-]
 call z,ContrastDown
; sub $1E                        ;$29 - [ON]
; call z,TurnOffCalc
 sub $25                        ;$30 - [ALPHA]
 call z,DeselectPiece
 sub $05                        ;$35 - [F1]
 ld b,a
 call z,ChangeTrayOrientation
 ld a,b
 sub $02                        ;$37 - [EXIT]
 jp z,ExitMenu
 dec a                          ;$38 - [MORE]
 jp z,SneakPeak

 in a,(3)
 bit 3,a                        ;check the [ON] key bit
 call z,TurnOffCalc             ;[ON] key was pressed

 ld a,%00111111
 out (1),a
 in a,(1)
 rla
 rla
 rla
 call c,SecondNotHeld
 call nc,SelectPiece

 ld hl,OutlineMoveCooldown
 ld a,(hl)
 or a
 jr z,$+5
 dec (hl)
 jr SkipCursorMove

 ld a,%01111110
 out (1),a
 in a,(1)
 rra
 ld b,a
KeyMod1 = $+1
 call nc,DummyFunction
;Self-Modified
; CursorDown for cursor selection mode
; OutlineDown for outline selection mode
 ld a,b
 rra
 ld b,a
KeyMod2 = $+1
 call nc,DummyFunction
;Self-Modified
; CursorLeft for cursor selection mode
; OutlineLeft for outline selection mode
 ld a,b
 rra
 ld b,a
KeyMod3 = $+1
 call nc,DummyFunction
;Self-Modified
; CursorRight for cursor selection mode
; OutlineRight for outline selection mode
 ld a,b
 rra
 ld b,a
KeyMod4 = $+1
 call nc,DummyFunction
;Self-Modified
; CursorUp for cursor selection mode
; OutlineUp for outline selection mode
SkipCursorMove:
;
;Check for Auto-Scroll
;
 ld a,(TrayVisible)
 or a
 jr z,NoCheckAutoScroll
 ld a,(CursorCoords+1)
 ld d,a
 or a
 call z,CheckAutoScrollDelay
 call z,CTBDecreaseOffset
 ld a,d
 cp 63
 call z,CheckAutoScrollDelay
 call z,CTBIncreaseOffset
NoCheckAutoScroll
 ld a,(NumSelected)
 or a
 call nz,DisplayInvertedPiece
 ld a,(TrayVisible)
 or a
 call nz,DisplayTray
 call PlaceCursor
 call CopyFromBuffer
 ld a,(PuzzleChange)
 or a
 call nz,CheckCompletion
 ld a,(Together)
 or a
 jp z,AssembleLoop
;
;Puzzle is together, tell user
;
 call StopTimer
 call RestoreScreen
 call CopyFromBuffer

 ld b,64
PuzzleTogetherAnim:
 push bc
 ld hl,PAGE1ADDR
 ld de,TempMem
 ld bc,16
 ldir
 ld hl,PAGE1ADDR+16
 ld de,PAGE1ADDR
 ld bc,1024-16
 ldir
 ld hl,TempMem
 ld de,PAGE1ADDR+1008
 ld bc,16
 ldir

 ld hl,PAGE2ADDR+1008
 ld de,TempMem
 ld bc,16
 ldir
 ld hl,PAGE2ADDR+1007
 ld de,PAGE2ADDR+1023
 ld bc,1024-16
 lddr
 ld hl,TempMem
 ld de,PAGE2ADDR
 ld bc,16
 ldir
 halt
 halt
 pop bc
 djnz PuzzleTogetherAnim

 call LoopTilEnter
 call ClearMem
 call CloseGray
 ld a,(NumPeaks)
 ld b,2
 ld de,CompleteText+52
 call NumToDec
 ld a,(TimerSecs)
 ld b,2
 ld de,CompleteText+34
 call NumToDec
 ld a,(TimerMins)
 ld b,2
 dec de
 call NumToDec
 ld a,(TimerHrs)
 ld b,2
 dec de
 call NumToDec
 ld hl,CompleteText
 call _PUTS
 call _PUTS
 call _PUTS
 call LoopTilEnter

 ld a,(PieceSize)
 add a,a
 add a,a
 ld b,a
 ld a,(SelectionMode)
 add a,a
 add a,b
 ld b,a
 ld a,(TrayStatus)
 add a,b
 ld b,a                 ;B = option combination offset (0-7)

 push bc
 push bc
;check if an entry exists
 add a,a
 add a,b
 ld b,a
 or a
 ld a,(HighScoreAddr)
 ld hl,(HighScoreAddr+1)
 jr z,$+7
 call _inc_ptr_ahl
 djnz $-3
 call _getb_ahl
 call _ram_page_1
 cp '-'
 jr z,NewHighScore+1    ;No entry, new high score
;compare times
 ld a,(HighScoreAddr)
 ld hl,(HighScoreAddr+1)
 ld b,26
 call _inc_ptr_ahl
 djnz $-3
 pop bc
 ld (StringTempStore),a
 ld a,b
 add a,a
 add a,b
 ld b,a
 or a
 ld a,(StringTempStore)
 jr z,$+7
 call _inc_ptr_ahl
 djnz $-3               ;AHL -> current highscore time

 ld de,TimerHrs
 ld b,3
CheckTimeLoop:
 push af
 push hl
 call _getb_ahl
 call _ram_page_1
 ld c,a
 ld a,(de)
 cp c
 jr c,NewHighScore+1        ;current stat is lower meaning new high score
 inc c
 cp c
 jp nc,DisplayHighScore-2   ;current stat is higher meaning no new high score
 dec de
 pop hl
 pop af
 call _dec_ptr_ahl
 djnz CheckTimeLoop
;compare peaks. times are equal (wow!)
 ld a,(HighScoreAddr)
 ld hl,(HighScoreAddr+1)
 ld b,48
 call _inc_ptr_ahl
 djnz $-3
 pop bc
 ld (StringTempStore),a
 ld a,b
 or a
 ld a,(StringTempStore)
 jr z,$+7
 call _inc_ptr_ahl
 djnz $-3
 call _getb_ahl
 call _ram_page_1
 ld c,a
 ld a,(NumPeaks)
 cp c
 jp nc,DisplayHighScore
 jr $+5

NewHighScore:
 pop hl
 pop hl
 pop hl
 call _clrLCD
 ld hl,HighScoreTxt
 call _PUTS
 call _PUTS
 set appTextSave,(iy+appflags)
 ld hl,$0805
 ld (_curRow),hl
 ld a,' '
 ld (_curUnder),a
 call _putc
 ld (_curRow),hl
 call _cursorOn
 ld hl,TempInitString
 ld de,StringTempStore
 ld bc,5
 ldir
 ld de,StringTempStore+1
GetInitLoop:
 push de
 call _getky
 pop de
 cp K_ENTER
 jr z,SaveHighScore
 cp K_DEL
 jr z,DeleteChar

 ld hl,KeyLetterTable
 ld bc,27
 cpir

 ld a,c
 or a
 jr z,GetInitLoop

 ld a,(de)
 or a
 jr z,GetInitLoop

 push de
 ld de,KeyLetterTable
 sbc hl,de
 ld a,l
 add a,'A'-1
 pop de
 ld (de),a
 inc de
 call _putc
CheckForMaxChar:
 ld a,(de)
 or a
 ld a,' '
 jr nz,EndGetInitLoop
MaxCharInput:
 ld a,$E7
EndGetInitLoop:
 ld (_curUnder),a
 call _putc
 ld hl,_curCol
 dec (hl)
 jr GetInitLoop

DeleteChar:
 dec de
 ld a,(de)
 inc de
 or a
 jr z,GetInitLoop
 dec de
 ld a,' '
 ld (de),a
 call _cursorOff
 ld hl,_curCol
 call _putc
 dec (hl)
 dec (hl)
 call _putc
 dec (hl)
 call _cursorOn
 jr GetInitLoop
;
TempInitString:
 .db 0,"   ",0
;
SaveHighScore:
 call _cursorOff
 res appTextSave,(iy+appflags)

 ld a,(PieceSize)
 add a,a
 add a,a
 ld b,a
 ld a,(SelectionMode)
 add a,a
 add a,b
 ld b,a
 ld a,(TrayStatus)
 add a,b
 ld b,a                 ;B = option combination offset (0-7)
;Save Inits
 push bc
 push bc
 add a,a
 add a,b
 ld b,a
 or a
 ld a,(HighScoreAddr)
 ld hl,(HighScoreAddr+1)
 jr z,$+7
 call _inc_ptr_ahl
 djnz $-3
 ld de,StringTempStore+1
 ld b,3
SaveInitsLoop:
 push af
 ld a,(de)
 ld c,a
 pop af
 call _writeb_inc_ahl
 inc de
 djnz SaveInitsLoop
;Save Time
 ld a,(HighScoreAddr)
 ld hl,(HighScoreAddr+1)
 ld b,24
 call _inc_ptr_ahl
 djnz $-3
 pop bc
 ld (StringTempStore),a
 ld a,b
 add a,a
 add a,b
 ld b,a
 or a
 ld a,(StringTempStore)
 jr z,$+7
 call _inc_ptr_ahl
 djnz $-3
 ld de,TimerSecs
 ld b,3
SaveTimeLoop:
 push af
 ld a,(de)
 ld c,a
 pop af
 call _writeb_inc_ahl
 inc de
 djnz SaveTimeLoop
;Save Peaks
 ld a,(HighScoreAddr)
 ld hl,(HighScoreAddr+1)
 ld b,48
 call _inc_ptr_ahl
 djnz $-3
 pop bc
 ld (StringTempStore),a
 ld a,b
 or a
 ld a,(StringTempStore)
 jr z,$+7
 call _inc_ptr_ahl
 djnz $-3
 push af
 ld a,(NumPeaks)
 ld c,a
 pop af
 call _writeb_inc_ahl
 jr DisplayHighScore
;
;Show the high score table
;
 pop hl
 pop hl
DisplayHighScore:
 ld a,(HighScoreAddr)
 ld hl,(HighScoreAddr+1)
 ld de,HighScoreInits+2
 ld b,8
InsertInitsLoop:
 ld c,a
 push hl
 call _getb_ahl
 ld (de),a
 inc de
 pop hl
 ld a,c
 call _inc_ptr_ahl
 ld c,a
 push hl
 call _getb_ahl
 ld (de),a
 inc de
 pop hl
 ld a,c
 call _inc_ptr_ahl
 ld c,a
 push hl
 call _getb_ahl
 ld (de),a
 inc de
 inc de
 inc de
 inc de
 pop hl
 ld a,c
 call _inc_ptr_ahl
 djnz InsertInitsLoop

 ld de,HighScoreTimes+9
 ld b,8
InsertTimesLoop:
 push bc
 push de
 ld c,a
 push hl
 call _getb_ahl
 ld b,2
 call NumToDec
 dec de
 pop hl
 ld a,c
 call _inc_ptr_ahl
 ld c,a
 push hl
 call _getb_ahl
 ld b,2
 call NumToDec
 dec de
 pop hl
 ld a,c
 call _inc_ptr_ahl
 ld c,a
 push hl
 call _getb_ahl
 ld b,2
 call NumToDec
 pop de
 ld a,c

 pop hl
 ld bc,11
 add hl,bc
 ex de,hl

 call _inc_ptr_ahl
 pop bc
 djnz InsertTimesLoop

 ld de,HighScorePeaks+3
 ld b,8
InsertPeaksLoop:
 push bc
 ld c,a
 push hl
 call _getb_ahl
 ld b,2
 call NumToDec
 ld hl,7
 add hl,de
 ex de,hl
 pop hl
 ld a,c
 call _inc_ptr_ahl
 pop bc
 djnz InsertPeaksLoop

 call _clrLCD
 ld hl,HighScoreTableTxt
 ld b,32
 call _VPUTS
 djnz $-3
 call LoopTilEnter


EndGame:
 call StopTimer
 call CloseGray
 jp CheckFile
DummyFunction:
 ret
EndProgram:
 call CloseGray
;
 call _clrLCD
 ld hl,_textShadow
 ld de,_textShadow+1
 ld (hl),' '
 ld bc,128
 ldir
;Clear the screen so that weird characters do not show.
 res onInterrupt,(IY+onflags)
 set graphdraw,(IY+graphflags)
;Set flags to clean graph and so that the user does not get
; an ON error on exit.
 set appTextSave,(iy+appflags)
;Make text save to text shadow again.
 ld hl,RandSeedSave
 ld de,$CA00+1018
 ld bc,6
 ldir
;Restore random seen
;
;Save all data necessary for user options and puzzle saving.
;
 ld hl,_asapvar
 rst 20h
 rst 10h
 ex de,hl
 ld a,b
 ld de,OptionsSave-$D748+4
 add hl,de
 adc a,0
 ld de,OptionsSave
RepCopy:
 push af
 push hl
 call _getb_ahl
 ld a,(de)
 ld (hl),a
 pop hl
 pop af
 call _inc_ptr_ahl
 inc de
 push hl
 ld hl,ResumeDataEnd
 or a
 sbc hl,de
 pop hl
 jr nz,RepCopy

 ld hl,_asapvar
 ld (hl),0
;Force the program to be reloaded into memory on next execution.
 jp _homeup
;
#include "JigCur.h"
#include "JigPiece.h"
#include "JigTray.h"
#include "JigMisc.h"
#include "JigData.h"
#include "DispStr.h"
#include "PutSprite.h"
#include "rle.h"
;gray.h MUST be included last or addresses beyond it will be wrong
#include "gray.h"

.end
